@@ -1,18 +1,16 @@
-from __future__ import absolute_import
-
 import sys
 import types
 
-from collections import OrderedDict
-
 import numpy as np
+import pandas as pd
 
 from .. import util
 from ..dimension import Dimension, asdim, dimension_name
-from ..ndmapping import NdMapping, item_check, sorted_context
 from ..element import Element
+from ..ndmapping import NdMapping, item_check, sorted_context
 from .grid import GridInterface
-from .interface import Interface, DataError, dask_array_module
+from .interface import DataError, Interface
+from .util import dask_array_module, finite_range
 
 
 def is_cupy(array):
@@ -65,7 +63,7 @@ class XArrayInterface(GridInterface):
         else:
             array = dataset.data[dataset.vdims[0].name]
         if not gridded:
-            return (np.product(array.shape, dtype=np.intp), len(dataset.dimensions()))
+            return (np.prod(array.shape, dtype=np.intp), len(dataset.dimensions()))
         shape_map = dict(zip(array.dims, array.shape))
         return tuple(shape_map.get(kd.name, np.nan) for kd in dataset.kdims[::-1])
 
@@ -82,7 +80,9 @@ class XArrayInterface(GridInterface):
             dim = asdim(dim)
             coord = data[dim.name]
             unit = coord.attrs.get('units') if dim.unit is None else dim.unit
-            if 'long_name' in coord.attrs:
+            if isinstance(unit, tuple):
+                unit = unit[0]
+            if isinstance(coord.attrs.get("long_name"), str):
                 spec = (dim.name, coord.attrs['long_name'])
             else:
                 spec = (dim.name, dim.label)
@@ -102,25 +102,28 @@ class XArrayInterface(GridInterface):
                 vdim.unit = data.attrs.get('units')
                 vdim.nodata = data.attrs.get('NODATA')
                 label = data.attrs.get('long_name')
-                if 'long_name' in data.attrs:
+                if isinstance(label, str):
                     vdim.label = label
             elif len(vdim_param.default) == 1:
-                vdim = vdim_param.default[0]
+                vdim = asdim(vdim_param.default[0])
                 if vdim.name in data.dims:
                     raise DataError("xarray DataArray does not define a name, "
-                                    "and the default of '%s' clashes with a "
+                                    f"and the default of '{vdim.name}' clashes with a "
                                     "coordinate dimension. Give the DataArray "
-                                    "a name or supply an explicit value dimension."
-                                    % vdim.name, cls)
+                                    "a name or supply an explicit value dimension.", cls)
             else:
                 raise DataError("xarray DataArray does not define a name "
-                                "and %s does not define a default value "
+                                f"and {eltype.__name__} does not define a default value "
                                 "dimension. Give the DataArray a name or "
-                                "supply an explicit vdim." % eltype.__name__,
+                                "supply an explicit vdim.",
                                 cls)
             if not packed:
-                vdims = [vdim]
-                data = data.to_dataset(name=vdim.name)
+                if vdim in data.dims:
+                    data = data.to_dataset(vdim.name)
+                    vdims = [asdim(vd) for vd in data.data_vars]
+                else:
+                    vdims = [vdim]
+                    data = data.to_dataset(name=vdim.name)
 
         if not isinstance(data, (xr.Dataset, xr.DataArray)):
             if kdims is None:
@@ -142,7 +145,7 @@ class XArrayInterface(GridInterface):
                     packed = True
                 else:
                     data = {d: v for d, v in zip(dimensions, data)}
-            elif isinstance(data, list) and data == []:
+            elif isinstance(data, (list, np.ndarray)) and len(data) == 0:
                 dimensions = [d.name for d in kdims + vdims]
                 data = {d: np.array([]) for d in dimensions[:ndims]}
                 data.update({d: np.empty((0,) * ndims) for d in dimensions[ndims:]})
@@ -152,7 +155,7 @@ class XArrayInterface(GridInterface):
                     for d, values in data.items()}
             coord_dims = [data[kd.name].ndim for kd in kdims]
             dims = tuple('dim_%d' % i for i in range(max(coord_dims)))[::-1]
-            coords = OrderedDict()
+            coords = {}
             for kd in kdims:
                 coord_vals = data[kd.name]
                 if coord_vals.ndim > 1:
@@ -174,12 +177,18 @@ class XArrayInterface(GridInterface):
                     arrays[vdim.name] = arr
                 data = xr.Dataset(arrays)
         else:
+            # Started to warn in xarray 2023.12.0:
+            # The return type of `Dataset.dims` will be changed to return a
+            # set of dimension names in future, in order to be more consistent
+            # with `DataArray.dims`. To access a mapping from dimension names to
+            # lengths, please use `Dataset.sizes`.
+            data_info =  data.sizes if hasattr(data, "sizes") else data.dims
             if not data.coords:
-                data = data.assign_coords(**{k: range(v) for k, v in data.dims.items()})
+                data = data.assign_coords(**{k: range(v) for k, v in data_info.items()})
             if vdims is None:
                 vdims = list(data.data_vars)
             if kdims is None:
-                xrdims = list(data.dims)
+                xrdims = list(data_info)
                 xrcoords = list(data.coords)
                 kdims = [name for name in data.indexes.keys()
                          if isinstance(data[name].data, np.ndarray)]
@@ -203,9 +212,36 @@ class XArrayInterface(GridInterface):
             raise TypeError('Data must be be an xarray Dataset type.')
         elif not_found:
             raise DataError("xarray Dataset must define coordinates "
-                            "for all defined kdims, %s coordinates not found."
-                            % not_found, cls)
+                            f"for all defined kdims, {not_found} coordinates not found.", cls)
 
+        for vdim in vdims:
+            if packed:
+                continue
+            da = data[vdim.name]
+            # Do not enforce validation for irregular arrays since they
+            # not need to be canonicalized
+            if any(len(da.coords[c].shape) > 1 for c in da.coords):
+                continue
+            undeclared = []
+            for c in da.coords:
+                if c in kdims or len(da[c].shape) != 1 or da[c].shape[0] <= 1:
+                    # Skip if coord is declared, represents irregular coordinates or is constant
+                    continue
+                elif all(d in kdims for d in da[c].dims):
+                    continue # Skip if coord is alias for another dimension
+                elif any(all(d in da[kd.name].dims for d in da[c].dims) for kd in kdims):
+                    # Skip if all the dims on the coord are present on another coord
+                    continue
+                undeclared.append(c)
+            if undeclared and eltype.param.kdims.bounds[1] not in (0, None):
+                raise DataError(
+                    f'The coordinates on the {vdim.name!r} DataArray do not match the '
+                    'provided key dimensions (kdims). The following coords '
+                    f'were left unspecified: {undeclared!r}. If you are requesting a '
+                    'lower dimensional view such as a histogram cast '
+                    'the xarray to a columnar format using the .to_dataframe '
+                    'or .to_dask_dataframe methods before providing it to '
+                    'HoloViews.')
         return data, {'kdims': kdims, 'vdims': vdims}, {}
 
 
@@ -219,7 +255,7 @@ class XArrayInterface(GridInterface):
             if not_found:
                 raise DataError("Supplied data does not contain specified "
                                 "dimensions, the following dimensions were "
-                                "not found: %s" % repr(not_found), cls)
+                                f"not found: {not_found!r}", cls)
 
         # Check whether irregular (i.e. multi-dimensional) coordinate
         # array dimensionality matches
@@ -228,15 +264,14 @@ class XArrayInterface(GridInterface):
             if cls.irregular(dataset, kd):
                 irregular.append((kd, dataset.data[kd.name].dims))
         if irregular:
-            nonmatching = ['%s: %s' % (kd, dims) for kd, dims in irregular[1:]
+            nonmatching = [f'{kd}: {dims}' for kd, dims in irregular[1:]
                            if set(dims) != set(irregular[0][1])]
             if nonmatching:
-                nonmatching = ['%s: %s' % irregular[0]] + nonmatching
+                nonmatching = ['{}: {}'.format(*irregular[0])] + nonmatching
                 raise DataError("The dimensions of coordinate arrays "
                                 "on irregular data must match. The "
                                 "following kdims were found to have "
-                                "non-matching array dimensions:\n\n%s"
-                                % ('\n'.join(nonmatching)), cls)
+                                "non-matching array dimensions:\n\n{}".format('\n'.join(nonmatching)), cls)
 
     @classmethod
     def compute(cls, dataset):
@@ -250,12 +285,9 @@ class XArrayInterface(GridInterface):
     def range(cls, dataset, dimension):
         dimension = dataset.get_dimension(dimension, strict=True)
         dim = dimension.name
-        if dataset._binned and dimension in dataset.kdims:
+        edges = dataset._binned and dimension in dataset.kdims
+        if edges:
             data = cls.coords(dataset, dim, edges=True)
-            if data.dtype.kind == 'M':
-                dmin, dmax = data.min(), data.max()
-            else:
-                dmin, dmax = np.nanmin(data), np.nanmax(data)
         else:
             if cls.packed(dataset) and dim in dataset.vdims:
                 data = dataset.data.values[..., dataset.vdims.index(dim)]
@@ -264,10 +296,14 @@ class XArrayInterface(GridInterface):
             if dimension.nodata is not None:
                 data = cls.replace_value(data, dimension.nodata)
 
-            if len(data):
-                dmin, dmax = data.min().data, data.max().data
-            else:
-                dmin, dmax = np.NaN, np.NaN
+        if not len(data):
+            dmin, dmax = np.nan, np.nan
+        elif data.dtype.kind == 'M' or not edges:
+            dmin, dmax = data.min(), data.max()
+            if not edges:
+                dmin, dmax = dmin.data, dmax.data
+        else:
+            dmin, dmax = np.nanmin(data), np.nanmax(data)
 
         da = dask_array_module()
         if da and isinstance(dmin, da.Array):
@@ -278,7 +314,7 @@ class XArrayInterface(GridInterface):
             dmax = dmax[()]
         dmin = dmin if np.isscalar(dmin) or isinstance(dmin, util.datetime_types) else dmin.item()
         dmax = dmax if np.isscalar(dmax) or isinstance(dmax, util.datetime_types) else dmax.item()
-        return dmin, dmax
+        return finite_range(data, dmin, dmax)
 
 
     @classmethod
@@ -289,9 +325,8 @@ class XArrayInterface(GridInterface):
 
         invalid = [d for d in index_dims if dataset.data[d.name].ndim > 1]
         if invalid:
-            if len(invalid) == 1: invalid = "'%s'" % invalid[0]
-            raise ValueError("Cannot groupby irregularly sampled dimension(s) %s."
-                             % invalid)
+            if len(invalid) == 1: invalid = f"'{invalid[0]}'"
+            raise ValueError(f"Cannot groupby irregularly sampled dimension(s) {invalid}.")
 
         group_kwargs = {}
         if group_type != 'raw' and issubclass(group_type, Element):
@@ -304,7 +339,7 @@ class XArrayInterface(GridInterface):
         group_by = [d.name for d in index_dims]
         data = []
         if len(dimensions) == 1:
-            for k, v in dataset.data.groupby(index_dims[0].name):
+            for k, v in dataset.data.groupby(index_dims[0].name, squeeze=False):
                 if drop_dim:
                     v = v.to_dataframe().reset_index()
                 data.append((k, group_type(v, **group_kwargs)))
@@ -411,9 +446,10 @@ class XArrayInterface(GridInterface):
         Given a dataset object and data in the appropriate format for
         the interface, return a simple scalar.
         """
-        if (not cls.packed(dataset) and len(data.data_vars) == 1 and
-            len(data[dataset.vdims[0].name].shape) == 0):
-            return data[dataset.vdims[0].name].item()
+        if not cls.packed(dataset) and len(data.data_vars) == 1:
+            array = data[dataset.vdims[0].name].squeeze()
+            if len(array.shape) == 0:
+                return array.item()
         return data
 
 
@@ -501,7 +537,9 @@ class XArrayInterface(GridInterface):
         return dataset.data
 
     @classmethod
-    def sort(cls, dataset, by=[], reverse=False):
+    def sort(cls, dataset, by=None, reverse=False):
+        if by is None:
+            by = []
         return dataset
 
     @classmethod
@@ -534,6 +572,9 @@ class XArrayInterface(GridInterface):
 
     @classmethod
     def select(cls, dataset, selection_mask=None, **selection):
+        if selection_mask is not None:
+            return dataset.data.where(selection_mask, drop=True)
+
         validated = {}
         for k, v in selection.items():
             dim = dataset.get_dimension(k, strict=True)
@@ -560,13 +601,13 @@ class XArrayInterface(GridInterface):
 
         # Restore constant dimensions
         indexed = cls.indexed(dataset, selection)
-        dropped = OrderedDict((d.name, np.atleast_1d(data[d.name]))
+        dropped = dict((d.name, np.atleast_1d(data[d.name]))
                    for d in dataset.kdims
                    if not data[d.name].data.shape)
         if dropped and not indexed:
             data = data.expand_dims(dropped)
             # see https://github.com/pydata/xarray/issues/2891
-            # since we only exapanded on dimnesions of size 1
+            # since we only expanded on dimensions of size 1
             # we can monkeypatch the dataarray back to writeable.
             for d in data.values():
                 if hasattr(d.data, 'flags'):
@@ -591,13 +632,13 @@ class XArrayInterface(GridInterface):
 
     @classmethod
     def length(cls, dataset):
-        return np.product([len(dataset.data[d.name]) for d in dataset.kdims], dtype=np.intp)
+        return np.prod([len(dataset.data[d.name]) for d in dataset.kdims], dtype=np.intp)
 
     @classmethod
     def dframe(cls, dataset, dimensions):
         import xarray as xr
         if cls.packed(dataset):
-            bands = {vd.name: dataset.data[..., i].drop('band')
+            bands = {vd.name: dataset.data[..., i].drop_vars('band')
                      for i, vd in enumerate(dataset.vdims)}
             data = xr.Dataset(bands)
         else:
@@ -608,11 +649,13 @@ class XArrayInterface(GridInterface):
         return data
 
     @classmethod
-    def sample(cls, dataset, samples=[]):
+    def sample(cls, dataset, samples=None):
+        if samples is None:
+            samples = []
         names = [kd.name for kd in dataset.kdims]
         samples = [dataset.data.sel(**{k: [v] for k, v in zip(names, s)}).to_dataframe().reset_index()
                    for s in samples]
-        return util.pd.concat(samples)
+        return pd.concat(samples)
 
     @classmethod
     def add_dimension(cls, dataset, dimension, dim_pos, values, vdim):
@@ -632,12 +675,12 @@ class XArrayInterface(GridInterface):
         prev_coords = set.intersection(*[
             set(var.coords) for var in data.data_vars.values()
         ])
-        coords = OrderedDict()
+        coords = {}
         for k, v in new_data.items():
             if k not in dataset.kdims:
                 continue
             elif isinstance(v, xr.DataArray):
-                coords[k] = v.rename(**{v.name: k})
+                coords[k] = v.rename(**({v.name: k} if v.name != k else {}))
                 continue
             coord_vals = cls.coords(dataset, k)
             if not coord_vals.ndim > 1 and np.all(coord_vals[1:] < coord_vals[:-1]):
@@ -647,7 +690,7 @@ class XArrayInterface(GridInterface):
             data = data.assign_coords(**coords)
 
         dims = tuple(kd.name for kd in dataset.kdims[::-1])
-        vars = OrderedDict()
+        vars = {}
         for k, v in new_data.items():
             if k in dataset.kdims:
                 continue
@@ -664,7 +707,7 @@ class XArrayInterface(GridInterface):
                 data = data.assign(vars)
             used_coords = set.intersection(*[set(var.coords) for var in data.data_vars.values()])
         drop_coords =  set.symmetric_difference(used_coords, prev_coords)
-        return data.drop([c for c in drop_coords if c in data.coords]), list(drop_coords)
+        return data.drop_vars([c for c in drop_coords if c in data.coords]), list(drop_coords)
 
 
 Interface.register(XArrayInterface)
