@@ -3,26 +3,25 @@ Provides Dimension objects for tracking the properties of a value,
 axis or map dimension. Also supplies the Dimensioned abstract
 baseclass for classes that accept Dimension values.
 """
-from __future__ import unicode_literals
-
-import re
+import builtins
 import datetime as dt
+import re
 import weakref
-
-from operator import itemgetter
-from collections import defaultdict, Counter
-from itertools import chain
+from collections import Counter, defaultdict
+from collections.abc import Iterable
 from functools import partial
+from itertools import chain
+from operator import itemgetter
 
-import param
 import numpy as np
+import param
 
 from . import util
-from .accessors import Opts, Apply, Redim
-from .options import Store, Options, cleanup_custom_options
+from .accessors import Apply, Opts, Redim
+from .options import Options, Store, cleanup_custom_options
 from .pprint import PrettyPrinter
 from .tree import AttrTree
-from .util import basestring, OrderedDict, bytes_to_unicode, unicode
+from .util import bytes_to_unicode
 
 # Alias parameter support for pickle loading
 
@@ -41,8 +40,8 @@ def param_aliases(d):
     Warning: We want to keep pickle hacking to a minimum!
     """
     for old, new in ALIASES.items():
-        old_param = '_%s_param_value' % old
-        new_param = '_%s_param_value' % new
+        old_param = f'_{old}_param_value'
+        new_param = f'_{new}_param_value'
         if old_param in d:
             d[new_param] = d.pop(old_param)
     return d
@@ -58,14 +57,8 @@ def asdim(dimension):
         A Dimension object constructed from the dimension spec. No
         copy is performed if the input is already a Dimension.
     """
-    if isinstance(dimension, Dimension):
-        return dimension
-    elif isinstance(dimension, (tuple, dict, basestring)):
-        return Dimension(dimension)
-    else:
-        raise ValueError('%s type could not be interpreted as Dimension. '
-                         'Dimensions must be declared as a string, tuple, '
-                         'dictionary or Dimension type.')
+    return dimension if isinstance(dimension, Dimension) else Dimension(dimension)
+
 
 def dimension_name(dimension):
     """Return the Dimension.name for a dimension-like object.
@@ -79,7 +72,7 @@ def dimension_name(dimension):
     """
     if isinstance(dimension, Dimension):
         return dimension.name
-    elif isinstance(dimension, basestring):
+    elif isinstance(dimension, str):
         return dimension
     elif isinstance(dimension, tuple):
         return dimension[0]
@@ -88,10 +81,9 @@ def dimension_name(dimension):
     elif dimension is None:
         return None
     else:
-        raise ValueError('%s type could not be interpreted as Dimension. '
+        raise ValueError(f'{type(dimension).__name__} type could not be interpreted as Dimension. '
                          'Dimensions must be declared as a string, tuple, '
-                         'dictionary or Dimension type.'
-                         % type(dimension).__name__)
+                         'dictionary or Dimension type.')
 
 
 def process_dimensions(kdims, vdims):
@@ -113,18 +105,15 @@ def process_dimensions(kdims, vdims):
     for group, dims in [('kdims', kdims), ('vdims', vdims)]:
         if dims is None:
             continue
-        elif isinstance(dims, (tuple, basestring, Dimension, dict)):
+        elif isinstance(dims, (tuple, str, Dimension, dict)):
             dims = [dims]
         elif not isinstance(dims, list):
-            raise ValueError("%s argument expects a Dimension or list of dimensions, "
-                             "specified as tuples, strings, dictionaries or Dimension "
-                             "instances, not a %s type. Ensure you passed the data as the "
-                             "first argument." % (group, type(dims).__name__))
-        for dim in dims:
-            if not isinstance(dim, (tuple, basestring, Dimension, dict)):
-                raise ValueError('Dimensions must be defined as a tuple, '
-                                 'string, dictionary or Dimension instance, '
-                                 'found a %s type.' % type(dim).__name__)
+            raise ValueError(
+                f"{group} argument expects a Dimension or list of dimensions, "
+                "specified as tuples, strings, dictionaries or Dimension "
+                f"instances, not a {type(dims).__name__} type. "
+                "Ensure you passed the data as the first argument."
+            )
         dimensions[group] = [asdim(d) for d in dims]
     return dimensions
 
@@ -191,8 +180,13 @@ class Dimension(param.Parameterized):
         maximum allowed value (defined by the range parameter) is
         continuous with the minimum allowed value.""")
 
-    value_format = param.Callable(default=None, doc="""
-        Formatting function applied to each value before display.""")
+    default = param.Parameter(default=None, doc="""
+        Default value of the Dimension which may be useful for widget
+        or other situations that require an initial or default value.""")
+
+    nodata = param.Integer(default=None, doc="""
+        Optional missing-data value for integer data.
+        If non-None, data with this value will be replaced with NaN.""")
 
     range = param.Tuple(default=(None, None), doc="""
         Specifies the minimum and maximum allowed values for a
@@ -202,25 +196,24 @@ class Dimension(param.Parameterized):
         Specifies a minimum and maximum reference value, which
         may be overridden by the data.""")
 
-    type = param.Parameter(default=None, doc="""
-        Optional type associated with the Dimension values. The type
-        may be an inbuilt constructor (such as int, str, float) or a
-        custom class object.""")
-
-    default = param.Parameter(default=None, doc="""
-        Default value of the Dimension which may be useful for widget
-        or other situations that require an initial or default value.""")
-
     step = param.Number(default=None, doc="""
         Optional floating point step specifying how frequently the
         underlying space should be sampled. May be used to define a
         discrete sampling over the range.""")
 
+    type = param.Parameter(default=None, doc="""
+        Optional type associated with the Dimension values. The type
+        may be an inbuilt constructor (such as int, str, float) or a
+        custom class object.""")
+
     unit = param.String(default=None, allow_None=True, doc="""
         Optional unit string associated with the Dimension. For
         instance, the string 'm' may be used represent units of meters
         and 's' to represent units of seconds.""")
 
+    value_format = param.Callable(default=None, doc="""
+        Formatting function applied to each value before display.""")
+
     values = param.List(default=[], doc="""
         Optional specification of the allowed value set for the
         dimension that may also be used to retain a categorical
@@ -239,61 +232,66 @@ class Dimension(param.Parameterized):
         if 'name' in params:
             raise KeyError('Dimension name must only be passed as the positional argument')
 
+        all_params = {}
         if isinstance(spec, Dimension):
-            existing_params = dict(spec.param.get_param_values())
-        elif (spec, params.get('unit', None)) in self.presets.keys():
-            preset = self.presets[(str(spec), str(params['unit']))]
-            existing_params = dict(preset.param.get_param_values())
-        elif isinstance(spec, dict):
-            existing_params = spec
-        elif spec in self.presets:
-            existing_params = dict(self.presets[spec].param.get_param_values())
-        elif (spec,) in self.presets:
-            existing_params = dict(self.presets[(spec,)].param.get_param_values())
-        else:
-            existing_params = {}
-
-        all_params = dict(existing_params, **params)
-        if isinstance(spec, tuple):
-            if not all(isinstance(s, basestring) for s in spec) or len(spec) != 2:
-                raise ValueError("Dimensions specified as a tuple must be a tuple "
-                                 "consisting of the name and label not: %s" % str(spec))
-            name, label = spec
-            all_params['name'] = name
-            all_params['label'] = label
-            if 'label' in params and (label != params['label']):
-                if params['label'] != label:
-                    self.param.warning(
-                        'Using label as supplied by keyword ({!r}), ignoring '
-                        'tuple value {!r}'.format(params['label'], label))
-                all_params['label'] = params['label']
-        elif isinstance(spec, basestring):
+            all_params.update(spec.param.values())
+        elif isinstance(spec, str):
+            if (spec, params.get('unit', None)) in self.presets.keys():
+                preset = self.presets[(str(spec), str(params['unit']))]
+                all_params.update(preset.param.values())
+            elif spec in self.presets:
+                all_params.update(self.presets[spec].param.values())
+            elif (spec,) in self.presets:
+                all_params.update(self.presets[(spec,)].param.values())
             all_params['name'] = spec
-            all_params['label'] = params.get('label', spec)
+            all_params['label'] = spec
+        elif isinstance(spec, tuple):
+            try:
+                all_params['name'], all_params['label'] = spec
+            except ValueError as exc:
+                raise ValueError(
+                    "Dimensions specified as a tuple must be a tuple "
+                    f"consisting of the name and label not: {spec}"
+                ) from exc
+            if 'label' in params and params['label'] != all_params['label']:
+                self.param.warning(
+                    f'Using label as supplied by keyword ({params["label"]!r}), '
+                    f'ignoring tuple value {all_params["label"]!r}')
+        elif isinstance(spec, dict):
+            all_params.update(spec)
+            try:
+                all_params.setdefault('label', spec['name'])
+            except KeyError as exc:
+                raise ValueError(
+                    'Dimension specified as a dict must contain a "name" key'
+                ) from exc
+        else:
+            raise ValueError(
+                f'{type(spec).__name__} type could not be interpreted as Dimension.  Dimensions must be '
+                'declared as a string, tuple, dictionary or Dimension type.'
+            )
+        all_params.update(params)
 
-        if all_params['name'] == '':
-            raise ValueError('Dimension name cannot be the empty string')
-        if all_params['label'] in ['', None]:
-            raise ValueError('Dimension label cannot be None or the empty string')
+        if not all_params['name']:
+            raise ValueError('Dimension name cannot be empty')
+        if not all_params['label']:
+            raise ValueError('Dimension label cannot be empty')
 
         values = params.get('values', [])
-        if isinstance(values, basestring) and values == 'initial':
+        if isinstance(values, str) and values == 'initial':
             self.param.warning("The 'initial' string for dimension values "
                                "is no longer supported.")
             values = []
 
         all_params['values'] = list(util.unique_array(values))
-        super(Dimension, self).__init__(**all_params)
+        super().__init__(**all_params)
         if self.default is not None:
             if self.values and self.default not in values:
-                raise ValueError('%r default %s not found in declared values: %s' %
-                                 (self, self.default, self.values))
+                raise ValueError(f'{self!r} default {self.default} not found in declared values: {self.values}')
             elif (self.range != (None, None) and
                   ((self.range[0] is not None and self.default < self.range[0]) or
                    (self.range[0] is not None and self.default > self.range[1]))):
-                raise ValueError('%r default %s not in declared range: %s' %
-                                 (self, self.default, self.range))
-
+                raise ValueError(f'{self!r} default {self.default} not in declared range: {self.range}')
 
     @property
     def spec(self):
@@ -304,13 +302,6 @@ class Dimension(param.Parameterized):
         """
         return (self.name, self.label)
 
-
-    def __call__(self, spec=None, **overrides):
-        self.param.warning('Dimension.__call__ method has been deprecated, '
-                           'use the clone method instead.')
-        return self.clone(spec=spec, **overrides)
-
-
     def clone(self, spec=None, **overrides):
         """Clones the Dimension with new parameters
 
@@ -324,17 +315,17 @@ class Dimension(param.Parameterized):
         Returns:
             Cloned Dimension object
         """
-        settings = dict(self.param.get_param_values(), **overrides)
+        settings = dict(self.param.values(), **overrides)
 
         if spec is None:
             spec = (self.name, overrides.get('label', self.label))
-        if 'label' in overrides and isinstance(spec, basestring) :
+        if 'label' in overrides and isinstance(spec, str) :
             spec = (spec, overrides['label'])
         elif 'label' in overrides and isinstance(spec, tuple) :
             if overrides['label'] != spec[1]:
                 self.param.warning(
-                    'Using label as supplied by keyword ({!r}), ignoring '
-                    'tuple value {!r}'.format(overrides['label'], spec[1]))
+                    f'Using label as supplied by keyword ({overrides["label"]!r}), '
+                    f'ignoring tuple value {spec[1]!r}')
             spec = (spec[0],  overrides['label'])
         return self.__class__(spec, **{k:v for k,v in settings.items()
                                        if k not in ['name', 'label']})
@@ -348,14 +339,15 @@ class Dimension(param.Parameterized):
         """
         Compatibility for pickles before alias attribute was introduced.
         """
-        super(Dimension, self).__setstate__(d)
-        self.label = self.name
+        super().__setstate__(d)
+        if '_label_param_value' not in d:
+            self.label = self.name
 
     def __eq__(self, other):
         "Implements equals operator including sanitized comparison."
 
         if isinstance(other, Dimension):
-            return self.spec == other.spec
+            return self.label == other.label
 
         # For comparison to strings. Name may be sanitized.
         return other in [self.name, self.label, util.dimension_sanitizer(self.name)]
@@ -382,17 +374,17 @@ class Dimension(param.Parameterized):
         return bytes_to_unicode(self.label) + bytes_to_unicode(unit)
 
     def pprint(self):
-        changed = dict(self.param.get_param_values(onlychanged=True))
-        if len(set([changed.get(k, k) for k in ['name','label']])) == 1:
-            return 'Dimension({spec})'.format(spec=repr(self.name))
+        changed = self.param.values(onlychanged=True)
+        if len({changed.get(k, k) for k in ['name','label']}) == 1:
+            return f'Dimension({self.name!r})'
 
         params = self.param.objects('existing')
         ordering = sorted(
             sorted(changed.keys()), key=lambda k: (
                 -float('inf') if params[k].precedence is None
                 else params[k].precedence))
-        kws = ", ".join('%s=%r' % (k, changed[k]) for k in ordering if k != 'name')
-        return 'Dimension({spec}, {kws})'.format(spec=repr(self.name), kws=kws)
+        kws = ", ".join(f'{k}={changed[k]!r}' for k in ordering if k != 'name')
+        return f'Dimension({self.name!r}, {kws})'
 
 
     def pprint_value(self, value, print_unit=False):
@@ -410,7 +402,7 @@ class Dimension(param.Parameterized):
         if formatter:
             if callable(formatter):
                 formatted_value = formatter(value)
-            elif isinstance(formatter, basestring):
+            elif isinstance(formatter, str):
                 if isinstance(value, (dt.datetime, dt.date)):
                     formatted_value = value.strftime(formatter)
                 elif isinstance(value, np.datetime64):
@@ -420,7 +412,7 @@ class Dimension(param.Parameterized):
                 else:
                     formatted_value = formatter % value
         else:
-            formatted_value = unicode(bytes_to_unicode(value))
+            formatted_value = str(bytes_to_unicode(value))
 
         if print_unit and self.unit is not None:
             formatted_value = formatted_value + ' ' + bytes_to_unicode(self.unit)
@@ -494,7 +486,7 @@ class LabelledData(param.Parameterized):
 
         self._id = None
         self.id = id
-        self._plot_id = plot_id or util.builtins.id(self)
+        self._plot_id = plot_id or builtins.id(self)
         if isinstance(params.get('label',None), tuple):
             (alias, long_name) = params['label']
             util.label_sanitizer.add_aliases(**{alias:long_name})
@@ -505,13 +497,11 @@ class LabelledData(param.Parameterized):
             util.group_sanitizer.add_aliases(**{alias:long_name})
             params['group'] = long_name
 
-        super(LabelledData, self).__init__(**params)
+        super().__init__(**params)
         if not util.group_sanitizer.allowable(self.group):
-            raise ValueError("Supplied group %r contains invalid characters." %
-                             self.group)
+            raise ValueError(f"Supplied group {self.group!r} contains invalid characters.")
         elif not util.label_sanitizer.allowable(self.label):
-            raise ValueError("Supplied label %r contains invalid characters." %
-                             self.label)
+            raise ValueError(f"Supplied label {self.label!r} contains invalid characters.")
 
     @property
     def id(self):
@@ -548,7 +538,7 @@ class LabelledData(param.Parameterized):
         Returns:
             Cloned object
         """
-        params = dict(self.param.get_param_values())
+        params = self.param.values()
         if new_type is None:
             clone_type = self.__class__
         else:
@@ -717,7 +707,7 @@ class LabelledData(param.Parameterized):
                                             if obj_dict['_id'] in s}
             else:
                 obj_dict['_id'] = None
-        except:
+        except Exception:
             self.param.warning("Could not pickle custom style information.")
         return obj_dict
 
@@ -726,37 +716,27 @@ class LabelledData(param.Parameterized):
         "Restores options applied to this object."
         d = param_aliases(d)
 
-        # Backwards compatibility for objects before id was made a property
-        opts_id = d['_id'] if '_id' in d else d.pop('id', None)
-        try:
-            load_options = Store.load_counter_offset is not None
-            if load_options:
-                matches = [k for k in d if k.startswith('_custom_option')]
-                for match in matches:
-                    custom_id = int(match.split('_')[-1])+Store.load_counter_offset
-                    if not isinstance(d[match], dict):
-                        # Backward compatibility before multiple backends
-                        backend_info = {'matplotlib':d[match]}
-                    else:
-                        backend_info = d[match]
-                    for backend, info in  backend_info.items():
-                        if backend not in Store._custom_options:
-                            Store._custom_options[backend] = {}
-                        Store._custom_options[backend][custom_id] = info
-                    if backend_info:
-                        if custom_id not in Store._weakrefs:
-                            Store._weakrefs[custom_id] = []
-                        ref = weakref.ref(self, partial(cleanup_custom_options, custom_id))
-                        Store._weakrefs[opts_id].append(ref)
-                    d.pop(match)
+        load_options = Store.load_counter_offset is not None
+        if load_options:
+            matches = [k for k in d if k.startswith('_custom_option')]
+            for match in matches:
+                custom_id = int(match.split('_')[-1])+Store.load_counter_offset
+                for backend, info in d[match].items():
+                    if backend not in Store._custom_options:
+                        Store._custom_options[backend] = {}
+                    Store._custom_options[backend][custom_id] = info
+                if d[match]:
+                    if custom_id not in Store._weakrefs:
+                        Store._weakrefs[custom_id] = []
+                    ref = weakref.ref(self, partial(cleanup_custom_options, custom_id))
+                    Store._weakrefs[d["_id"]].append(ref)
+                d.pop(match)
+
+            if d["_id"] is not None:
+                d["_id"] += Store.load_counter_offset
 
-                if opts_id is not None:
-                    opts_id += Store.load_counter_offset
-        except:
-            self.param.warning("Could not unpickle custom style information.")
-        d['_id'] = opts_id
         self.__dict__.update(d)
-        super(LabelledData, self).__setstate__({})
+        super().__setstate__(d)
 
 
 class Dimensioned(LabelledData):
@@ -811,7 +791,7 @@ class Dimensioned(LabelledData):
     by the value dimensions and ending with the deep dimensions.
     """
 
-    cdims = param.Dict(default=OrderedDict(), doc="""
+    cdims = param.Dict(default={}, doc="""
        The constant dimensions defined as a dictionary of Dimension:value
        pairs providing additional dimension information about the object.
 
@@ -846,10 +826,10 @@ class Dimensioned(LabelledData):
         if 'cdims' in params:
             params['cdims'] = {d if isinstance(d, Dimension) else Dimension(d): val
                                for d, val in params['cdims'].items()}
-        super(Dimensioned, self).__init__(data, **params)
+        super().__init__(data, **params)
         self.ndims = len(self.kdims)
         cdims = [(d.name, val) for d, val in self.cdims.items()]
-        self._cached_constants = OrderedDict(cdims)
+        self._cached_constants = dict(cdims)
         self._settings = None
 
         # Instantiate accessors
@@ -879,7 +859,7 @@ class Dimensioned(LabelledData):
         for dim in dimensions:
             if isinstance(dim, Dimension): dim = dim.name
             if dim not in self.kdims:
-                raise Exception("Supplied dimensions %s not found." % dim)
+                raise Exception(f"Supplied dimensions {dim} not found.")
             valid_dimensions.append(dim)
         return valid_dimensions
 
@@ -929,15 +909,15 @@ class Dimensioned(LabelledData):
                     for dim in getattr(self, group)]
         elif isinstance(selection, list):
             dims =  [dim for group in selection
-                     for dim in getattr(self, '%sdims' % aliases.get(group))]
+                     for dim in getattr(self, f'{aliases.get(group)}dims')]
         elif aliases.get(selection) in lambdas:
             selection = aliases.get(selection, selection)
             lmbd, kwargs = lambdas[selection]
             key_traversal = self.traverse(lmbd, **kwargs)
             dims = [dim for keydims in key_traversal for dim in keydims]
         else:
-            raise KeyError("Invalid selection %r, valid selections include"
-                           "'all', 'value' and 'key' dimensions" % repr(selection))
+            raise KeyError(f"Invalid selection {repr(selection)!r}, valid selections include"
+                           "'all', 'value' and 'key' dimensions")
         return [(dim.label if label == 'long' else dim.name)
                 if label else dim for dim in dims]
 
@@ -953,23 +933,23 @@ class Dimensioned(LabelledData):
         Returns:
             Dimension object for the requested dimension or default
         """
-        if dimension is not None and not isinstance(dimension, (int, basestring, Dimension)):
+        if dimension is not None and not isinstance(dimension, (int, str, Dimension)):
             raise TypeError('Dimension lookup supports int, string, '
                             'and Dimension instances, cannot lookup '
-                            'Dimensions using %s type.' % type(dimension).__name__)
+                            f'Dimensions using {type(dimension).__name__} type.')
         all_dims = self.dimensions()
         if isinstance(dimension, int):
             if 0 <= dimension < len(all_dims):
                 return all_dims[dimension]
             elif strict:
-                raise KeyError("Dimension %r not found" % dimension)
+                raise KeyError(f"Dimension {dimension!r} not found")
             else:
                 return default
 
         if isinstance(dimension, Dimension):
             dims = [d for d in all_dims if dimension == d]
             if strict and not dims:
-                raise KeyError("%r not found." % dimension)
+                raise KeyError(f"{dimension!r} not found.")
             elif dims:
                 return dims[0]
             else:
@@ -981,7 +961,7 @@ class Dimensioned(LabelledData):
             name_map.update({dim.label: dim for dim in all_dims})
             name_map.update({util.dimension_sanitizer(dim.name): dim for dim in all_dims})
             if strict and dimension not in name_map:
-                raise KeyError("Dimension %r not found." % dimension)
+                raise KeyError(f"Dimension {dimension!r} not found.")
             else:
                 return name_map.get(dimension, default)
 
@@ -1004,10 +984,9 @@ class Dimensioned(LabelledData):
         dim = dimension_name(dimension)
         try:
             dimensions = self.kdims+self.vdims
-            return [i for i, d in enumerate(dimensions) if d == dim][0]
-        except IndexError:
-            raise Exception("Dimension %s not found in %s." %
-                            (dim, self.__class__.__name__))
+            return next(i for i, d in enumerate(dimensions) if d == dim)
+        except StopIteration:
+            raise Exception(f"Dimension {dim} not found in {self.__class__.__name__}.") from None
 
 
     def get_dimension_type(self, dim):
@@ -1163,8 +1142,7 @@ class Dimensioned(LabelledData):
         if val:
             return np.array([val])
         else:
-            raise Exception("Dimension %s not found in %s." %
-                            (dimension, self.__class__.__name__))
+            raise Exception(f"Dimension {dimension} not found in {self.__class__.__name__}.")
 
 
     def range(self, dimension, data_range=True, dimension_range=True):
@@ -1196,33 +1174,18 @@ class Dimensioned(LabelledData):
                 ranges = self.traverse(range_fn, [match_fn])
                 lower, upper = util.max_range(ranges)
         else:
-            lower, upper = (np.NaN, np.NaN)
+            lower, upper = (np.nan, np.nan)
         if not dimension_range:
             return lower, upper
         return util.dimension_range(lower, upper, dimension.range, dimension.soft_range)
 
-
     def __repr__(self):
         return PrettyPrinter.pprint(self)
 
     def __str__(self):
         return repr(self)
 
-    def __unicode__(self):
-        return unicode(PrettyPrinter.pprint(self))
-
-    def __call__(self, options=None, **kwargs):
-        self.param.warning(
-            'Use of __call__ to set options will be deprecated '	
-            'in the next major release (1.14.0). Use the equivalent .opts '
-            'method instead.')	
-
-        if not kwargs and options is None:	
-            return self.opts.clear()
-
-        return self.opts(options, **kwargs)
-
-    def options(self, *args, **kwargs):
+    def options(self, *args, clone=True, **kwargs):
         """Applies simplified option definition returning a new object.
 
         Applies options on an object or nested group of objects in a
@@ -1261,11 +1224,10 @@ class Dimensioned(LabelledData):
             Returns the cloned object with the options applied
         """
         backend = kwargs.get('backend', None)
-        clone = kwargs.pop('clone', True)
 
-        if len(args) == 0 and len(kwargs)==0:
+        if not (args or kwargs):
             options = None
-        elif args and isinstance(args[0], basestring):
+        elif args and isinstance(args[0], str):
             options = {args[0]: kwargs}
         elif args and isinstance(args[0], list):
             if kwargs:
@@ -1299,7 +1261,8 @@ class Dimensioned(LabelledData):
 
         obj = self
         for backend, expanded in expanded_backends:
-            obj = obj.opts._dispatch_opts(expanded, backend=backend, clone=clone)
+            if expanded is not None:
+                obj = obj.opts._dispatch_opts(expanded, backend=backend, clone=clone)
         return obj
 
     def _repr_mimebundle_(self, include=None, exclude=None):
@@ -1350,24 +1313,19 @@ class ViewableTree(AttrTree, Dimensioned):
         AttrTree.__init__(self, items, identifier, parent, **kwargs)
         Dimensioned.__init__(self, self.data, **params)
 
-
-    @classmethod
-    def from_values(cls, vals):
-        "Deprecated method to construct tree from list of objects"
-        name = cls.__name__        
-        param.main.param.warning("%s.from_values is deprecated, the %s "
-                                 "constructor may now be used directly."
-                                 % (name, name))
-        return cls(items=cls._process_items(vals))
-
-
     @classmethod
     def _process_items(cls, vals):
         "Processes list of items assigning unique paths to each."
+        from .layout import AdjointLayout
+
         if type(vals) is cls:
             return vals.data
-        elif not isinstance(vals, (list, tuple)):
+        elif isinstance(vals, (AdjointLayout, str)):
+            # `string` vals isn't supported but checked anyway
+            # for better exception message.
             vals = [vals]
+        elif isinstance(vals, Iterable):
+            vals = list(vals)
         items = []
         counts = defaultdict(lambda: 1)
         cls._unpack_paths(vals, items, counts)
@@ -1395,7 +1353,7 @@ class ViewableTree(AttrTree, Dimensioned):
 
         new_items = []
         counts = defaultdict(lambda: 0)
-        for i, (path, item) in enumerate(items):
+        for path, item in items:
             if counter[path] > 1:
                 path = path + (util.int_to_roman(counts[path]+1),)
             else:
@@ -1460,7 +1418,7 @@ class ViewableTree(AttrTree, Dimensioned):
             vals = np.concatenate(values)
             return vals if expanded else util.unique_array(vals)
         else:
-            return super(ViewableTree, self).dimension_values(
+            return super().dimension_values(
                 dimension, expanded, flat)
 
     def __len__(self):
