@@ -2,23 +2,20 @@ import os
 from unittest import SkipTest
 
 import param
-import holoviews
-
-from IPython import version_info
 from IPython.core.completer import IPCompleter
 from IPython.display import HTML, publish_display_data
 from param import ipython as param_ext
 
-from ..core.dimension import LabelledData
-from ..core.tree import AttrTree
-from ..core.options import Store
-from ..element.comparison import ComparisonTestCase
-from ..util import extension
-from ..plotting.renderer import Renderer
-from .magics import load_magics
-from .display_hooks import display  # noqa (API import)
-from .display_hooks import pprint_display, png_display, svg_display
+import holoviews as hv
 
+from ..core.dimension import LabelledData
+from ..core.options import Store
+from ..core.tree import AttrTree
+from ..element.comparison import ComparisonTestCase
+from ..plotting.renderer import Renderer
+from ..util import extension
+from .display_hooks import display, png_display, pprint_display, svg_display
+from .magics import load_magics
 
 AttrTree._disabled_prefixes = ['_repr_','_ipython_canary_method_should_not_exist']
 
@@ -37,16 +34,17 @@ class IPTestCase(ComparisonTestCase):
     """
 
     def setUp(self):
-        super(IPTestCase, self).setUp()
+        super().setUp()
         try:
             import IPython
             from IPython.display import HTML, SVG
             self.ip = IPython.InteractiveShell()
             if self.ip is None:
                 raise TypeError()
-        except Exception:
-                raise SkipTest("IPython could not be started")
+        except Exception as e:
+            raise SkipTest("IPython could not be started") from e
 
+        self.ip.displayhook.flush = lambda: None  # To avoid gc.collect called in it
         self.addTypeEqualityFunc(HTML, self.skip_comparison)
         self.addTypeEqualityFunc(SVG,  self.skip_comparison)
 
@@ -55,7 +53,7 @@ class IPTestCase(ComparisonTestCase):
     def get_object(self, name):
         obj = self.ip._object_find(name).obj
         if obj is None:
-            raise self.failureException("Could not find object %s" % name)
+            raise self.failureException(f"Could not find object {name}")
         return obj
 
 
@@ -83,8 +81,8 @@ class notebook_extension(extension):
 
     logo = param.Boolean(default=True, doc="Toggles display of HoloViews logo")
 
-    inline = param.Boolean(default=True, doc="""
-        Whether to inline JS and CSS resources. 
+    inline = param.Boolean(default=False, doc="""
+        Whether to inline JS and CSS resources.
         If disabled, resources are loaded from CDN if one is available.""")
 
     width = param.Number(default=None, bounds=(0, 100), doc="""
@@ -96,11 +94,11 @@ class notebook_extension(extension):
         format will be displayed).
 
         Although the 'html' format is supported across backends, other
-        formats supported by the current backend (e.g 'png' and 'svg'
+        formats supported by the current backend (e.g. 'png' and 'svg'
         using the matplotlib backend) may be used. This may be useful to
-        export figures to other formats such as PDF with nbconvert. """)
+        export figures to other formats such as PDF with nbconvert.""")
 
-    allow_jedi_completion = param.Boolean(default=False, doc="""
+    allow_jedi_completion = param.Boolean(default=True, doc="""
        Whether to allow jedi tab-completion to be enabled in IPython.
        Disabled by default because many HoloViews features rely on
        tab-completion machinery not supported when using jedi.""")
@@ -109,25 +107,27 @@ class notebook_extension(extension):
        Whether to monkey patch IPython to use the correct tab-completion
        behavior. """)
 
+    enable_mathjax = param.Boolean(default=False, doc="""
+        Whether to load bokeh-mathjax bundle in the notebook.""")
+
     _loaded = False
 
     def __call__(self, *args, **params):
-        super(notebook_extension, self).__call__(*args, **params)
+        comms = params.pop('comms', None)
+        super().__call__(*args, **params)
         # Abort if IPython not found
         try:
             ip = params.pop('ip', None) or get_ipython() # noqa (get_ipython)
-        except:
+        except Exception:
             return
 
         # Notebook archive relies on display hooks being set to work.
         try:
-            if version_info[0] >= 4:
-                import nbformat # noqa (ensures availability)
-            else:
-                from IPython import nbformat # noqa (ensures availability)
+            import nbformat  # noqa: F401
+
             try:
                 from .archive import notebook_archive
-                holoviews.archive = notebook_archive
+                hv.archive = notebook_archive
             except AttributeError as e:
                 if str(e) != "module 'tornado.web' has no attribute 'asynchronous'":
                     raise
@@ -138,7 +138,7 @@ class notebook_extension(extension):
         # Not quite right, should be set when switching backends
         if 'matplotlib' in Store.renderers and not notebook_extension._loaded:
             svg_exporter = Store.renderers['matplotlib'].instance(holomap=None,fig='svg')
-            holoviews.archive.exporters = [svg_exporter] + holoviews.archive.exporters
+            hv.archive.exporters = [svg_exporter] + hv.archive.exporters
 
         p = param.ParamOverrides(self, {k:v for k,v in params.items() if k!='config'})
         if p.case_sensitive_completion:
@@ -153,8 +153,8 @@ class notebook_extension(extension):
         if 'html' not in p.display_formats and len(p.display_formats) > 1:
             msg = ('Output magic unable to control displayed format '
                    'as IPython notebook uses fixed precedence '
-                   'between %r' % p.display_formats)
-            display(HTML('<b>Warning</b>: %s' % msg))
+                   f'between {p.display_formats!r}')
+            display(HTML(f'<b>Warning</b>: {msg}'))
 
         loaded = notebook_extension._loaded
         if loaded == False:
@@ -168,9 +168,9 @@ class notebook_extension(extension):
 
         css = ''
         if p.width is not None:
-            css += '<style>div.container { width: %s%% }</style>' % p.width
+            css += f'<style>div.container {{ width: {p.width}% }}</style>'
         if p.css:
-            css += '<style>%s</style>' % p.css
+            css += f'<style>{p.css}</style>'
 
         if css:
             display(HTML(css))
@@ -178,20 +178,29 @@ class notebook_extension(extension):
         resources = list(resources)
         if len(resources) == 0: return
 
+        from panel import config, extension as panel_extension
+        if hasattr(config, 'comms') and comms:
+            config.comms = comms
+
+        same_cell_execution = published = getattr(self, '_repeat_execution_in_cell', False)
         for r in [r for r in resources if r != 'holoviews']:
             Store.renderers[r].load_nb(inline=p.inline)
-        Renderer.load_nb()
+        Renderer.load_nb(inline=p.inline, reloading=same_cell_execution, enable_mathjax=p.enable_mathjax)
+
+        if not published and hasattr(panel_extension, "_display_globals"):
+            panel_extension._display_globals()
 
         if hasattr(ip, 'kernel') and not loaded:
             Renderer.comm_manager.get_client_comm(notebook_extension._process_comm_msg,
                                                   "hv-extension-comm")
 
         # Create a message for the logo (if shown)
-        self.load_hvjs(logo=p.logo,
-                       bokeh_logo=  p.logo and ('bokeh' in resources),
-                       mpl_logo=    p.logo and (('matplotlib' in resources)
-                                                or resources==['holoviews']),
-                       plotly_logo= p.logo and ('plotly' in resources))
+        if not same_cell_execution and p.logo:
+            self.load_logo(logo=p.logo,
+                           bokeh_logo=  p.logo and ('bokeh' in resources),
+                           mpl_logo=    p.logo and (('matplotlib' in resources)
+                                                    or resources==['holoviews']),
+                           plotly_logo= p.logo and ('plotly' in resources))
 
     @classmethod
     def completions_sorting_key(cls, word):
@@ -201,11 +210,11 @@ class notebook_extension(extension):
         elif word.startswith('_'): prio1 = 1
         if word.endswith('='):     prio1 = -1
         if word.startswith('%%'):
-            if not "%" in word[2:]:
-                word = word[2:];   prio2 = 2
+            if '%' not in word[2:]:
+                word, prio2 = word[2:], 2
         elif word.startswith('%'):
-            if not "%" in word[1:]:
-                word = word[1:];   prio2 = 1
+            if '%' not in word[1:]:
+                word, prio2 = word[1:], 1
         return prio1, word, prio2
 
 
@@ -230,8 +239,7 @@ class notebook_extension(extension):
 
         unmatched_args = set(args) - set(resources)
         if unmatched_args:
-            display(HTML('<b>Warning:</b> Unrecognized resources %s'
-                         % ', '.join(unmatched_args)))
+            display(HTML("<b>Warning:</b> Unrecognized resources '{}'".format("', '".join(unmatched_args))))
 
         resources = [r for r in resources if r not in disabled]
         if ('holoviews' not in disabled) and ('holoviews' not in resources):
@@ -239,10 +247,9 @@ class notebook_extension(extension):
         return resources
 
     @classmethod
-    def load_hvjs(cls, logo=False, bokeh_logo=False, mpl_logo=False, plotly_logo=False,
-                  JS=True, message='HoloViewsJS successfully loaded.'):
+    def load_logo(cls, logo=False, bokeh_logo=False, mpl_logo=False, plotly_logo=False):
         """
-        Displays javascript and CSS to initialize HoloViews widgets.
+        Allow to display Holoviews' logo and the plotting extensions' logo.
         """
         import jinja2
 
@@ -252,8 +259,7 @@ class notebook_extension(extension):
         html = template.render({'logo':        logo,
                                 'bokeh_logo':  bokeh_logo,
                                 'mpl_logo':    mpl_logo,
-                                'plotly_logo': plotly_logo,
-                                'message':     message})
+                                'plotly_logo': plotly_logo})
         publish_display_data(data={'text/html': html})
 
 
