@@ -1,28 +1,35 @@
 from collections import defaultdict
 
-import param
 import numpy as np
+import param
 from bokeh.models import (
-    StaticLayoutProvider, NodesAndLinkedEdges, EdgesAndLinkedNodes,
-    Patches, Bezier, ColumnDataSource, NodesOnly
+    Bezier,
+    ColumnDataSource,
+    EdgesAndLinkedNodes,
+    NodesAndLinkedEdges,
+    NodesOnly,
+    Patches,
+    StaticLayoutProvider,
 )
 
 from ...core.data import Dataset
 from ...core.options import Cycle, abbreviated_exception
 from ...core.util import dimension_sanitizer, unique_array
-from ...element import Graph
 from ...util.transform import dim
-from ..mixins import ChordMixin
-from ..util import process_cmap, get_directed_graph_paths
+from ..mixins import ChordMixin, GraphMixin
+from ..util import get_directed_graph_paths, process_cmap
 from .chart import ColorbarPlot, PointPlot
 from .element import CompositeElementPlot, LegendPlot
 from .styles import (
-    base_properties, line_properties, fill_properties, text_properties,
-    rgba_tuple
+    base_properties,
+    fill_properties,
+    line_properties,
+    rgba_tuple,
+    text_properties,
 )
 
 
-class GraphPlot(CompositeElementPlot, ColorbarPlot, LegendPlot):
+class GraphPlot(GraphMixin, CompositeElementPlot, ColorbarPlot, LegendPlot):
 
     arrowhead_length = param.Number(default=0.025, doc="""
       If directed option is enabled this determines the length of the
@@ -87,21 +94,13 @@ class GraphPlot(CompositeElementPlot, ColorbarPlot, LegendPlot):
             dims = element.nodes.dimensions()
             dims = [(dims[2].pprint_label, '@{index_hover}')]+dims[3:]
         elif self.inspection_policy == 'edges':
-            kdims = [(kd.pprint_label, '@{%s_values}' % kd)
+            kdims = [(kd.pprint_label, f'@{{{kd}_values}}')
                      if kd in ('start', 'end') else kd for kd in element.kdims]
             dims = kdims+element.vdims
         else:
             dims = []
         return dims, {}
 
-    def get_extents(self, element, ranges, range_type='combined'):
-        return super().get_extents(element.nodes, ranges, range_type)
-
-    def _get_axis_dims(self, element):
-        if isinstance(element, Graph):
-            element = element.nodes
-        return element.dimensions()[:2]
-
     def _get_edge_colors(self, element, ranges, edge_data, edge_mapping, style):
         cdim = element.get_dimension(self.edge_color_index)
         if not cdim:
@@ -165,8 +164,8 @@ class GraphPlot(CompositeElementPlot, ColorbarPlot, LegendPlot):
                                  "Expected %d, found %d paths." % (len(element), len(edges)))
         elif self.directed:
             xdim, ydim = element.nodes.kdims[:2]
-            x_range = ranges[xdim.name]['combined']
-            y_range = ranges[ydim.name]['combined']
+            x_range = ranges[xdim.label]['combined']
+            y_range = ranges[ydim.label]['combined']
             arrow_len = np.hypot(y_range[1]-y_range[0], x_range[1]-x_range[0])*self.arrowhead_length
             arrows = get_directed_graph_paths(element, arrow_len)
             path_data['xs'] = [arr[:, 0] for arr in arrows]
@@ -187,12 +186,13 @@ class GraphPlot(CompositeElementPlot, ColorbarPlot, LegendPlot):
         if nodes.dtype.kind not in 'uif':
             node_indices = {v: i for i, v in enumerate(nodes)}
             index = np.array([node_indices[n] for n in nodes], dtype=np.int32)
-            layout = {str(node_indices[k]): (y, x) if self.invert_axes else (x, y)
+            layout = {node_indices[k]: (y, x) if self.invert_axes else (x, y)
                       for k, (x, y) in zip(nodes, node_positions)}
         else:
             index = nodes.astype(np.int32)
-            layout = {str(k): (y, x) if self.invert_axes else (x, y)
+            layout = {k: (y, x) if self.invert_axes else (x, y)
                       for k, (x, y) in zip(index, node_positions)}
+
         point_data = {'index': index}
 
         # Handle node colors
@@ -512,7 +512,7 @@ class TriMeshPlot(GraphPlot):
             z = element.nodes.dimension_values(vertex_dim)
             z = z[simplices].mean(axis=1)
             element = element.add_dimension(vertex_dim, len(element.vdims), z, vdim=True)
-        element.edgepaths
+        element._initialize_edgepaths()
         return element
 
     def _init_glyphs(self, plot, element, ranges, source):
