@@ -1,37 +1,67 @@
-from itertools import groupby
 from collections import defaultdict
+from itertools import groupby
 
 import numpy as np
 import param
-
 from bokeh.layouts import gridplot
 from bokeh.models import (
-    ColumnDataSource, Column, Row, Div, Title, Legend, Axis, ColorBar
+    Axis,
+    ColorBar,
+    Column,
+    ColumnDataSource,
+    Div,
+    Legend,
+    Row,
+    Title,
 )
-from bokeh.models.widgets import Panel, Tabs
+from bokeh.models.layouts import TabPanel, Tabs
 
-from ...selection import NoOpSelectionDisplay
 from ...core import (
-    OrderedDict, Store, AdjointLayout, NdLayout, Layout, Empty,
-    GridSpace, HoloMap, Element
+    AdjointLayout,
+    Element,
+    Empty,
+    GridSpace,
+    HoloMap,
+    Layout,
+    NdLayout,
+    Store,
 )
 from ...core.options import SkipRendering
 from ...core.util import (
-    cftime_to_timestamp, cftime_types, get_method_owner,
-    is_param_method, unique_iterator, wrap_tuple, wrap_tuple_streams,
-    _STANDARD_CALENDARS
+    _STANDARD_CALENDARS,
+    cftime_to_timestamp,
+    cftime_types,
+    get_method_owner,
+    is_param_method,
+    unique_iterator,
+    wrap_tuple,
+    wrap_tuple_streams,
 )
+from ...selection import NoOpSelectionDisplay
 from ..links import Link
 from ..plot import (
-    DimensionedPlot, GenericCompositePlot, GenericLayoutPlot,
-    GenericElementPlot, GenericOverlayPlot, GenericAdjointLayoutPlot,
-    CallbackPlot
+    CallbackPlot,
+    DimensionedPlot,
+    GenericAdjointLayoutPlot,
+    GenericCompositePlot,
+    GenericElementPlot,
+    GenericLayoutPlot,
+    GenericOverlayPlot,
 )
-from ..util import attach_streams, displayable, collate
+from ..util import attach_streams, collate, displayable
 from .links import LinkCallback
 from .util import (
-    filter_toolboxes, make_axis, update_shared_sources, empty_plot,
-    decode_bytes, theme_attr_json, cds_column_replace, get_default
+    cds_column_replace,
+    decode_bytes,
+    empty_plot,
+    filter_toolboxes,
+    get_default,
+    make_axis,
+    merge_tools,
+    select_legends,
+    sync_legends,
+    theme_attr_json,
+    update_shared_sources,
 )
 
 
@@ -124,16 +154,15 @@ class BokehPlot(DimensionedPlot, CallbackPlot):
         new_data = {}
         for k, values in data.items():
             values = decode_bytes(values) # Bytes need decoding to strings
-
             # Certain datetime types need to be converted
             if len(values) and isinstance(values[0], cftime_types):
                 if any(v.calendar not in _STANDARD_CALENDARS for v in values):
                     self.param.warning(
                         'Converting cftime.datetime from a non-standard '
-                        'calendar (%s) to a standard calendar for plotting. '
+                        f'calendar ({values[0].calendar}) to a standard calendar for plotting. '
                         'This may lead to subtle errors in formatting '
                         'dates, for accurate tick formatting switch to '
-                        'the matplotlib backend.' % values[0].calendar)
+                        'the matplotlib backend.')
                 values = cftime_to_timestamp(values, 'ms')
             new_data[k] = values
         return new_data
@@ -143,9 +172,6 @@ class BokehPlot(DimensionedPlot, CallbackPlot):
         """
         Update datasource with data for a new frame.
         """
-        if not self.document:
-            return
-
         data = self._postprocess_data(data)
         empty = all(len(v) == 0 for v in data.values())
         if (self.streaming and self.streaming[0].data is self.current_frame.data
@@ -267,7 +293,8 @@ class BokehPlot(DimensionedPlot, CallbackPlot):
         if 'title' in self.handles:
             title_div = self.handles['title']
         else:
-            title_div = Div(width=width, style={"white-space": "nowrap"})  # so it won't wrap long titles easily
+            # so it won't wrap long titles easily
+            title_div = Div(width=width, styles={"white-space": "nowrap"})
         title_div.text = title_tags
 
         return title_div
@@ -303,8 +330,6 @@ class BokehPlot(DimensionedPlot, CallbackPlot):
                         renderer.update(data_source=new_source)
                     else:
                         renderer.update(source=new_source)
-                    if hasattr(renderer, 'view'):
-                        renderer.view.update(source=new_source)
                     plot.handles['source'] = plot.handles['cds'] = new_source
                     plots.append(plot)
                 shared_sources.append(new_source)
@@ -324,11 +349,11 @@ class BokehPlot(DimensionedPlot, CallbackPlot):
             cb = Link._callbacks['bokeh'][type(link)]
             if src_plot is None or (link._requires_target and tgt_plot is None):
                 continue
+            # The link callback (`cb`) is instantiated (with side-effects).
             callbacks.append(cb(self.root, link, src_plot, tgt_plot))
         return callbacks
 
 
-
 class CompositePlot(BokehPlot):
     """
     CompositePlot is an abstract baseclass for plot types that draw
@@ -462,6 +487,9 @@ class GridPlot(CompositePlot, GenericCompositePlot):
         as a tuple specifying width and height or an integer for a
         square plot.""")
 
+    sync_legends = param.Boolean(default=True, doc="""
+        Whether to sync the legend when muted/unmuted based on the name""")
+
     def __init__(self, layout, ranges=None, layout_num=1, keys=None, **params):
         if not isinstance(layout, GridSpace):
             raise Exception("GridPlot only accepts GridSpace.")
@@ -485,10 +513,10 @@ class GridPlot(CompositePlot, GenericCompositePlot):
         else:
             width, height = self.plot_size, self.plot_size
 
-        subplots = OrderedDict()
+        subplots = {}
         frame_ranges = self.compute_ranges(layout, None, ranges)
         keys = self.keys[:1] if self.dynamic else self.keys
-        frame_ranges = OrderedDict([(key, self.compute_ranges(layout, key, frame_ranges))
+        frame_ranges = dict([(key, self.compute_ranges(layout, key, frame_ranges))
                                     for key in keys])
         collapsed_layout = layout.clone(shared_data=False, id=layout.id)
         for i, coord in enumerate(layout.keys(full_grid=True)):
@@ -545,8 +573,8 @@ class GridPlot(CompositePlot, GenericCompositePlot):
             if plotting_class is None:
                 if view is not None:
                     self.param.warning(
-                        "Bokeh plotting class for %s type not found, "
-                        "object will not be rendered." % vtype.__name__)
+                        f"Bokeh plotting class for {vtype.__name__} type not found, "
+                        "object will not be rendered.")
             else:
                 subplot = plotting_class(view, dimensions=self.dimensions,
                                          show_title=False, subplot=True,
@@ -560,7 +588,9 @@ class GridPlot(CompositePlot, GenericCompositePlot):
         return subplots, collapsed_layout
 
 
-    def initialize_plot(self, ranges=None, plots=[]):
+    def initialize_plot(self, ranges=None, plots=None):
+        if plots is None:
+            plots = []
         ranges = self.compute_ranges(self.layout, self.keys[-1], None)
         passed_plots = list(plots)
         plots = [[None for c in range(self.cols)] for r in range(self.rows)]
@@ -575,11 +605,15 @@ class GridPlot(CompositePlot, GenericCompositePlot):
             else:
                 passed_plots.append(None)
 
-        plot = gridplot(plots[::-1], merge_tools=self.merge_tools,
+        plot = gridplot(plots[::-1],
+                        merge_tools=False,
                         sizing_mode=self.sizing_mode,
                         toolbar_location=self.toolbar)
+        if self.sync_legends:
+            sync_legends(plot)
         plot = self._make_axes(plot)
-
+        if hasattr(plot, "toolbar") and self.merge_tools:
+            plot.toolbar = merge_tools(plots, hide_toolbar=True)
         title = self._get_title_div(self.keys[-1])
         if title:
             plot = Column(title, plot)
@@ -629,7 +663,9 @@ class GridPlot(CompositePlot, GenericCompositePlot):
             if self.shared_yaxis:
                 x_axis.margin = (0, 0, 0, 50)
                 r1, r2 = r1[::-1], r2[::-1]
-            plot = gridplot([r1, r2])
+            plot = gridplot([r1, r2], merge_tools=False)
+            if self.merge_tools:
+                plot.toolbar = merge_tools([r1, r2])
         elif y_axis:
             models = [y_axis, plot]
             if self.shared_yaxis: models = models[::-1]
@@ -672,6 +708,24 @@ class LayoutPlot(CompositePlot, GenericLayoutPlot):
     merge_tools = param.Boolean(default=True, doc="""
         Whether to merge all the tools into a single toolbar""")
 
+    sync_legends = param.Boolean(default=True, doc="""
+        Whether to sync the legend when muted/unmuted based on the name""")
+
+    show_legends = param.ClassSelector(default=None, class_=(list, int, bool), doc="""
+        Whether to show the legend for a particular subplot by index. If True all legends
+        will be shown. If False no legends will be shown.""")
+
+    legend_position = param.ObjectSelector(objects=["top_right",
+                                                    "top_left",
+                                                    "bottom_left",
+                                                    "bottom_right",
+                                                    'right', 'left',
+                                                    'top', 'bottom'],
+                                                    default="top_right",
+                                                    doc="""
+        Allows selecting between a number of predefined legend position
+        options. Will only be applied if show_legend is not None.""")
+
     tabs = param.Boolean(default=False, doc="""
         Whether to display overlaid plots in separate panes""")
 
@@ -682,6 +736,11 @@ class LayoutPlot(CompositePlot, GenericLayoutPlot):
             self.traverse(lambda x: attach_streams(self, x.hmap, 2),
                           [GenericElementPlot])
 
+    @param.depends('show_legends', 'legend_position', watch=True, on_init=True)
+    def _update_show_legend(self):
+        if self.show_legends is not None:
+            select_legends(self.layout, self.show_legends, self.legend_position)
+
     def _init_layout(self, layout):
         # Situate all the Layouts in the grid and compute the gridspec
         # indices for all the axes required by each LayoutPlot.
@@ -689,7 +748,7 @@ class LayoutPlot(CompositePlot, GenericLayoutPlot):
         collapsed_layout = layout.clone(shared_data=False, id=layout.id)
         frame_ranges = self.compute_ranges(layout, None, None)
         keys = self.keys[:1] if self.dynamic else self.keys
-        frame_ranges = OrderedDict([(key, self.compute_ranges(layout, key, frame_ranges))
+        frame_ranges = dict([(key, self.compute_ranges(layout, key, frame_ranges))
                                     for key in keys])
         layout_items = layout.grid_items()
         layout_dimensions = layout.kdims if isinstance(layout, NdLayout) else None
@@ -712,7 +771,7 @@ class LayoutPlot(CompositePlot, GenericLayoutPlot):
             # to create the correct subaxes for all plots in the layout
             layout_key, _ = layout_items.get((r, c), (None, None))
             if isinstance(layout, NdLayout) and layout_key:
-                layout_dimensions = OrderedDict(zip(layout_dimensions, layout_key))
+                layout_dimensions = dict(zip(layout_dimensions, layout_key))
 
             # Generate the axes and create the subplots with the appropriate
             # axis objects, handling any Empty objects.
@@ -790,8 +849,8 @@ class LayoutPlot(CompositePlot, GenericLayoutPlot):
                 continue
             elif plot_type is None:
                 self.param.warning(
-                    "Bokeh plotting class for %s type not found, object "
-                    " will not be rendered." % vtype.__name__)
+                    f"Bokeh plotting class for {vtype.__name__} type not found, object "
+                    " will not be rendered.")
                 continue
             num = num if len(self.coords) > 1 else 0
             subplot = plot_type(element, keys=self.keys,
@@ -906,15 +965,14 @@ class LayoutPlot(CompositePlot, GenericLayoutPlot):
 
                     if nsubplots == 1:
                         grid = subplots[0]
-                    elif nsubplots == 2:
-                        grid = gridplot([subplots], merge_tools=self.merge_tools,
-                                        toolbar_location=self.toolbar,
-                                        sizing_mode=sizing_mode)
                     else:
-                        grid = [[subplots[2], None], subplots[:2]]
-                        grid = gridplot(children=grid, merge_tools=self.merge_tools,
+                        children = [subplots] if nsubplots == 2 else [[subplots[2], None], subplots[:2]]
+                        grid = gridplot(children,
+                                        merge_tools=False,
                                         toolbar_location=self.toolbar,
                                         sizing_mode=sizing_mode)
+                        if self.merge_tools:
+                            grid.toolbar = merge_tools(children)
                     tab_plots.append((title, grid))
                     continue
 
@@ -943,14 +1001,20 @@ class LayoutPlot(CompositePlot, GenericLayoutPlot):
         # Wrap in appropriate layout model
         if self.tabs:
             plots = filter_toolboxes([p for t, p in tab_plots])
-            panels = [Panel(child=child, title=t) for t, child in tab_plots]
+            panels = [TabPanel(child=child, title=t) for t, child in tab_plots]
             layout_plot = Tabs(tabs=panels, sizing_mode=sizing_mode)
         else:
             plot_grid = filter_toolboxes(plot_grid)
-            layout_plot = gridplot(children=plot_grid,
-                                   toolbar_location=self.toolbar,
-                                   merge_tools=self.merge_tools,
-                                   sizing_mode=sizing_mode)
+            layout_plot = gridplot(
+                children=plot_grid,
+                toolbar_location=self.toolbar,
+                merge_tools=False,
+                sizing_mode=sizing_mode
+            )
+            if self.sync_legends:
+                sync_legends(layout_plot)
+            if self.merge_tools:
+                layout_plot.toolbar = merge_tools(plot_grid)
 
         title = self._get_title_div(self.keys[-1])
         if title:
@@ -1002,7 +1066,7 @@ class AdjointLayoutPlot(BokehPlot, GenericAdjointLayoutPlot):
         # The supplied (axes, view) objects as indexed by position
         super().__init__(subplots=subplots, **params)
 
-    def initialize_plot(self, ranges=None, plots=[]):
+    def initialize_plot(self, ranges=None, plots=None):
         """
         Plot all the views contained in the AdjointLayout Object using axes
         appropriate to the layout configuration. All the axes are
@@ -1010,6 +1074,8 @@ class AdjointLayoutPlot(BokehPlot, GenericAdjointLayoutPlot):
         invoke subplots with correct options and styles and hide any
         empty axes as necessary.
         """
+        if plots is None:
+            plots = []
         if plots is None: plots = []
         adjoined_plots = []
         for pos in self.view_positions:
