@@ -1,55 +1,71 @@
-from __future__ import absolute_import, division, unicode_literals
-
-import re
-import time
-import sys
 import calendar
 import datetime as dt
-from types import FunctionType
-
+import re
+import time
 from collections import defaultdict
-from contextlib import contextmanager
+from contextlib import contextmanager, suppress
+from itertools import permutations
 
-import param
 import bokeh
 import numpy as np
-
-from bokeh.core.json_encoder import serialize_json # noqa (API import)
+import pandas as pd
+from bokeh.core.json_encoder import serialize_json  # noqa (API import)
+from bokeh.core.property.datetime import Datetime
 from bokeh.core.validation import silence
-from bokeh.layouts import WidgetBox, Row, Column
-from bokeh.models import tools
+from bokeh.layouts import Column, Row, group_tools
 from bokeh.models import (
-    Model, ToolbarBox, FactorRange, Range1d, Plot, Spacer, CustomJS,
-    GridBox, DatetimeAxis, CategoricalAxis
+    CategoricalAxis,
+    CopyTool,
+    CustomJS,
+    DataRange1d,
+    DatetimeAxis,
+    ExamineTool,
+    FactorRange,
+    FullscreenTool,
+    GridBox,
+    GridPlot,
+    LayoutDOM,
+    LinearAxis,
+    LogAxis,
+    MercatorAxis,
+    Model,
+    Plot,
+    Range1d,
+    SaveTool,
+    Spacer,
+    Tabs,
+    Toolbar,
+    tools,
 )
-from bokeh.models.formatters import (
-    FuncTickFormatter, TickFormatter, PrintfTickFormatter
-)
-from bokeh.models.widgets import DataTable, Tabs, Div
-from bokeh.plotting import Figure
+from bokeh.models.formatters import PrintfTickFormatter, TickFormatter
+from bokeh.models.scales import CategoricalScale, LinearScale, LogScale
+from bokeh.models.tickers import BasicTicker, FixedTicker, Ticker
+from bokeh.models.widgets import DataTable, Div
+from bokeh.plotting import figure
+from bokeh.themes import built_in_themes
 from bokeh.themes.theme import Theme
+from packaging.version import Version
 
-try:
-    from bokeh.themes import built_in_themes
-except:
-    built_in_themes = {}
-
-try:
-    from bkcharts import Chart
-except:
-    Chart = type(None) # Create stub for isinstance check
-
+from ...core import util
+from ...core.layout import Layout
 from ...core.ndmapping import NdMapping
-from ...core.overlay import Overlay
+from ...core.overlay import NdOverlay, Overlay
+from ...core.spaces import DynamicMap, get_nested_dmaps
 from ...core.util import (
-    LooseVersion, _getargspec, basestring, callable_name, cftime_types,
-    cftime_to_timestamp, pd, unique_array, isnumeric, arraylike_types
+    arraylike_types,
+    callable_name,
+    cftime_to_timestamp,
+    cftime_types,
+    isnumeric,
+    unique_array,
 )
-from ...core.spaces import get_nested_dmaps, DynamicMap
+from ...util.warnings import warn
 from ..util import dim_axis_label
 
-bokeh_version = LooseVersion(bokeh.__version__)  # noqa
-
+bokeh_version = Version(Version(bokeh.__version__).base_version)
+bokeh32 = bokeh_version >= Version("3.2")
+bokeh33 = bokeh_version >= Version("3.3")
+bokeh34 = bokeh_version >= Version("3.4")
 
 TOOL_TYPES = {
     'pan': tools.PanTool,
@@ -95,17 +111,25 @@ def convert_timestamp(timestamp):
     """
     Converts bokehJS timestamp to datetime64.
     """
-    datetime = dt.datetime.utcfromtimestamp(timestamp/1000.)
+    datetime = dt.datetime.fromtimestamp(timestamp/1000, tz=dt.timezone.utc)
     return np.datetime64(datetime.replace(tzinfo=None))
 
 
+def prop_is_none(value):
+    """
+    Checks if property value is None.
+    """
+    return (value is None or
+            (isinstance(value, dict) and 'value' in value
+             and value['value'] is None))
+
+
 def decode_bytes(array):
     """
     Decodes an array, list or tuple of bytestrings to avoid python 3
     bokeh serialization errors
     """
-    if (sys.version_info.major == 2 or not len(array) or
-        (isinstance(array, arraylike_types) and array.dtype.kind != 'O')):
+    if (not len(array) or (isinstance(array, arraylike_types) and array.dtype.kind != 'O')):
         return array
     decoded = [v.decode('utf-8') if isinstance(v, bytes) else v for v in array]
     if isinstance(array, np.ndarray):
@@ -133,9 +157,9 @@ def layout_padding(plots, renderer):
         for c, p in enumerate(row):
             if p is None:
                 p = empty_plot(widths[c], heights[r])
-            elif hasattr(p, 'plot_width') and p.plot_width == 0 and p.plot_height == 0:
-                p.plot_width = widths[c]
-                p.plot_height = heights[r]
+            elif hasattr(p, 'width') and p.width == 0 and p.height == 0:
+                p.width = widths[c]
+                p.height = heights[r]
             expanded_plots[r].append(p)
     return expanded_plots
 
@@ -143,21 +167,21 @@ def layout_padding(plots, renderer):
 def compute_plot_size(plot):
     """
     Computes the size of bokeh models that make up a layout such as
-    figures, rows, columns, widgetboxes and Plot.
+    figures, rows, columns, and Plot.
     """
-    if isinstance(plot, GridBox):
+    if isinstance(plot, (GridBox, GridPlot)):
         ndmapping = NdMapping({(x, y): fig for fig, y, x in plot.children}, kdims=['x', 'y'])
         cols = ndmapping.groupby('x')
         rows = ndmapping.groupby('y')
         width = sum([max([compute_plot_size(f)[0] for f in col]) for col in cols])
         height = sum([max([compute_plot_size(f)[1] for f in row]) for row in rows])
         return width, height
-    elif isinstance(plot, (Div, ToolbarBox)):
-        # Cannot compute size for Div or ToolbarBox
+    elif isinstance(plot, (Div, Toolbar)):
+        # Cannot compute size for Div or Toolbar
         return 0, 0
-    elif isinstance(plot, (Row, Column, WidgetBox, Tabs)):
+    elif isinstance(plot, (Row, Column, Tabs)):
         if not plot.children: return 0, 0
-        if isinstance(plot, Row) or (isinstance(plot, ToolbarBox) and plot.toolbar_location not in ['right', 'left']):
+        if isinstance(plot, Row) or (isinstance(plot, Toolbar) and plot.toolbar_location not in ['right', 'left']):
             w_agg, h_agg = (np.sum, np.max)
         elif isinstance(plot, Tabs):
             w_agg, h_agg = (np.max, np.max)
@@ -165,13 +189,13 @@ def compute_plot_size(plot):
             w_agg, h_agg = (np.max, np.sum)
         widths, heights = zip(*[compute_plot_size(child) for child in plot.children])
         return w_agg(widths), h_agg(heights)
-    elif isinstance(plot, (Figure, Chart)):
-        if plot.plot_width:
-            width = plot.plot_width
+    elif isinstance(plot, figure):
+        if plot.width:
+            width = plot.width
         else:
             width = plot.frame_width + plot.min_border_right + plot.min_border_left
-        if plot.plot_height:
-            height = plot.plot_height
+        if plot.height:
+            height = plot.height
         else:
             height = plot.frame_height + plot.min_border_bottom + plot.min_border_top
         return width, height
@@ -250,13 +274,12 @@ def compute_layout_properties(
                     sizing_mode = 'scale_height'
                 else:
                     sizing_mode = 'scale_both'
+            elif responsive == 'width':
+                sizing_mode = 'stretch_both'
+            elif responsive == 'height':
+                sizing_mode = 'stretch_height'
             else:
-                if responsive == 'width':
-                    sizing_mode = 'stretch_both'
-                elif responsive == 'height':
-                    sizing_mode = 'stretch_height'
-                else:
-                    sizing_mode = 'stretch_both'
+                sizing_mode = 'stretch_both'
 
 
     if fixed_aspect:
@@ -277,11 +300,11 @@ def compute_layout_properties(
                 aspect = None
                 if logger:
                     logger.warning(
-                        "%s value was ignored because absolute width and "
+                        f"{aspect_type} value was ignored because absolute width and "
                         "height values were provided. Either supply "
                         "explicit frame_width and frame_height to achieve "
                         "desired aspect OR supply a combination of width "
-                        "or height and an aspect value." % aspect_type)
+                        "or height and an aspect value.")
         elif fixed_width and responsive:
             height = None
             responsive = False
@@ -347,14 +370,137 @@ def compute_layout_properties(
                        'provide a numeric value, \'equal\' or '
                        '\'square\'.')
 
-    return ({'aspect_ratio': aspect_ratio,
-             'aspect_scale': aspect_scale,
-             'match_aspect': match_aspect,
-             'sizing_mode' : sizing_mode},
-            {'frame_width' : frame_width,
-             'frame_height': frame_height,
-             'plot_height' : height,
-             'plot_width'  : width})
+    aspect_info = {
+        'aspect_ratio': aspect_ratio,
+        'aspect_scale': aspect_scale,
+        'match_aspect': match_aspect,
+        'sizing_mode' : sizing_mode
+    }
+    dimension_info =  {
+        'frame_width' : frame_width,
+        'frame_height': frame_height,
+        'height' : height,
+        'width'  : width
+    }
+
+    return aspect_info, dimension_info
+
+
+def merge_tools(plot_grid, *, disambiguation_properties=None, hide_toolbar=False):
+    """
+    Merges tools defined on a grid of plots into a single toolbar.
+    All tools of the same type are merged unless they define one
+    of the disambiguation properties. By default `name`, `icon`, `tags`
+    and `description` can be used to prevent tools from being merged.
+    """
+    tools = []
+    for row in plot_grid:
+        for item in row:
+            if isinstance(item, LayoutDOM):
+                for p in item.select(dict(type=Plot)):
+                    tools.extend(p.toolbar.tools)
+            if hide_toolbar and hasattr(item, 'toolbar_location'):
+                item.toolbar_location = None
+            if isinstance(item, GridPlot):
+                item.toolbar_location = None
+
+    def merge(tool, group):
+        if issubclass(tool, (SaveTool, CopyTool, ExamineTool, FullscreenTool)):
+            return tool()
+        else:
+            return None
+
+    if not disambiguation_properties:
+        disambiguation_properties = {'name', 'icon', 'tags', 'description'}
+
+    ignore = set()
+    for tool in tools:
+        for p in tool.properties_with_values():
+            if p not in disambiguation_properties:
+                ignore.add(p)
+
+    return Toolbar(tools=group_tools(tools, merge=merge, ignore=ignore)) if tools else Toolbar()
+
+
+def sync_legends(bokeh_layout):
+    """This syncs the legends of all plots in a grid based on their name.
+
+    Parameters
+    ----------
+    bokeh_layout : bokeh.models.{GridPlot, Row, Column}
+        Gridplot to sync legends of.
+    """
+    if len(bokeh_layout.children) < 2:
+        return
+
+    # Collect all glyph with names
+    items = defaultdict(list)
+    click_policies = set()
+    for fig in bokeh_layout.children:
+        if isinstance(fig, tuple):  # GridPlot
+            fig = fig[0]
+        if not isinstance(fig, figure):
+            continue
+        for r in fig.renderers:
+            if r.name:
+                items[r.name].append(r)
+        if fig.legend:
+            click_policies.add(fig.legend[0].click_policy)
+
+    click_policies.discard("none")  # If legend is not visible, click_policy is "none"
+    if len(click_policies) > 1:
+        warn("Click policy of legends are not the same, no syncing will happen.")
+        return
+    elif not click_policies:
+        return
+
+    # Link all glyphs with the same name
+    mapping = {"mute": "muted", "hide": "visible"}
+    policy = mapping.get(next(iter(click_policies)))
+    code = f"dst.{policy} = src.{policy}"
+    for item in items.values():
+        for src, dst in permutations(item, 2):
+            src.js_on_change(
+                policy,
+                CustomJS(code=code, args=dict(src=src, dst=dst)),
+            )
+
+
+def select_legends(holoviews_layout, figure_index=None, legend_position="top_right"):
+    """ Only displays selected legends in plot layout.
+
+    Parameters
+    ----------
+    holoviews_layout : Holoviews Layout
+        Holoviews Layout with legends.
+    figure_index : list[int] | bool | int | None
+        Index of the figures which legends to show.
+        If None is chosen, only the first figures legend is shown
+        If True is chosen, all legends are shown.
+    legend_position : str
+        Position of the legend(s).
+    """
+    if figure_index is None:
+        figure_index = [0]
+    elif isinstance(figure_index, bool):
+        figure_index = range(len(holoviews_layout)) if figure_index else []
+    elif isinstance(figure_index, int):
+        figure_index = [figure_index]
+    if not isinstance(holoviews_layout, Layout):
+        holoviews_layout = [holoviews_layout]
+
+    for i, plot in enumerate(holoviews_layout):
+        if not isinstance(plot, (NdOverlay, Overlay)):
+            continue
+        if i in figure_index:
+            plot.opts(show_legend=True, legend_position=legend_position)
+        else:
+            plot.opts(show_legend=False)
+
+    if isinstance(holoviews_layout, list):
+        return holoviews_layout[0]
+
+    return holoviews_layout
 
 
 @contextmanager
@@ -394,10 +540,10 @@ def font_size_to_pixels(size):
     """
     Convert a fontsize to a pixel value
     """
-    if size is None or not isinstance(size, basestring):
+    if size is None or not isinstance(size, str):
         return
     conversions = {'em': 16, 'pt': 16/12.}
-    val = re.findall('\d+', size)
+    val = re.findall(r'\d+', size)
     unit = re.findall('[a-z]+', size)
     if (val and not unit) or (val and unit[0] == 'px'):
         return int(val[0])
@@ -436,17 +582,17 @@ def make_axis(axis, size, factors, dim, flip=False, rotation=0,
         height = int(axis_height + np.abs(np.sin(rotation)) *
                      ((nchars*tick_px)*0.82)) + tick_px + label_px
         opts = dict(x_axis_type='auto', x_axis_label=axis_label,
-                    x_range=ranges, y_range=ranges2, plot_height=height,
-                    plot_width=size)
+                    x_range=ranges, y_range=ranges2, height=height,
+                    width=size)
     else:
         # Adjust width to compensate for label rotation
         align = 'left' if flip else 'right'
         width = int(axis_height + np.abs(np.cos(rotation)) *
                     ((nchars*tick_px)*0.82)) + tick_px + label_px
         opts = dict(y_axis_label=axis_label, x_range=ranges2,
-                    y_range=ranges, plot_width=width, plot_height=size)
+                    y_range=ranges, height=size, width=width)
 
-    p = Figure(toolbar_location=None, tools=[], **opts)
+    p = figure(toolbar_location=None, tools=[], **opts)
     p.outline_line_alpha = 0
     p.grid.grid_line_alpha = 0
 
@@ -516,16 +662,16 @@ def pad_width(model, table_padding=0.85, tabs_padding=1.2):
     elif isinstance(model, Tabs):
         vals = [pad_width(t) for t in model.tabs]
         width = np.max([v for v in vals if v is not None])
-        for model in model.tabs:
-            model.width = width
+        for submodel in model.tabs:
+            submodel.width = width
             width = int(tabs_padding*width)
     elif isinstance(model, DataTable):
         width = model.width
         model.width = int(table_padding*width)
-    elif isinstance(model, (WidgetBox, Div)):
+    elif isinstance(model, Div):
         width = model.width
     elif model:
-        width = model.plot_width
+        width = model.width
     else:
         width = 0
     return width
@@ -534,7 +680,7 @@ def pad_width(model, table_padding=0.85, tabs_padding=1.2):
 def pad_plots(plots):
     """
     Accepts a grid of bokeh plots in form of a list of lists and
-    wraps any DataTable or Tabs in a WidgetBox with appropriate
+    wraps any DataTable or Tabs in a Column with appropriate
     padding. Required to avoid overlap in gridplot.
     """
     widths = []
@@ -544,7 +690,8 @@ def pad_plots(plots):
             width = pad_width(p)
             row_widths.append(width)
         widths.append(row_widths)
-    plots = [[WidgetBox(p, width=w) if isinstance(p, (DataTable, Tabs)) else p
+
+    plots = [[Column(p, width=w) if isinstance(p, (DataTable, Tabs)) else p
               for p, w in zip(row, ws)] for row, ws in zip(plots, widths)]
     return plots
 
@@ -556,39 +703,14 @@ def filter_toolboxes(plots):
     """
     if isinstance(plots, list):
         plots = [filter_toolboxes(plot) for plot in plots]
+    elif hasattr(plots, 'toolbar'):
+        plots.toolbar_location = None
     elif hasattr(plots, 'children'):
         plots.children = [filter_toolboxes(child) for child in plots.children
-                          if not isinstance(child, ToolbarBox)]
+                          if not isinstance(child, Toolbar)]
     return plots
 
 
-def py2js_tickformatter(formatter, msg=''):
-    """
-    Uses py2js to compile a python tick formatter to JS code
-    """
-    try:
-        from pscript import py2js
-    except ImportError:
-        param.main.param.warning(
-            msg+'Ensure pscript is installed ("conda install pscript" '
-            'or "pip install pscript")')
-        return
-    try:
-        jscode = py2js(formatter, 'formatter')
-    except Exception as e:
-        error = 'Pyscript raised an error: {0}'.format(e)
-        error = error.replace('%', '%%')
-        param.main.param.warning(msg+error)
-        return
-
-    args = _getargspec(formatter).args
-    arg_define = 'var %s = tick;' % args[0] if args else ''
-    return_js = 'return formatter();\n'
-    jsfunc = '\n'.join([arg_define, jscode, return_js])
-    match = re.search('(formatter \= function flx_formatter \(.*\))', jsfunc)
-    return jsfunc[:match.start()] + 'formatter = function ()' + jsfunc[match.end():]
-
-
 def get_tab_title(key, frame, overlay):
     """
     Computes a title for bokeh tabs from the key in the overlay, the
@@ -636,14 +758,14 @@ def filter_batched_data(data, mapping):
             if 'transform' in v:
                 continue
             v = v['field']
-        elif not isinstance(v, basestring):
+        elif not isinstance(v, str):
             continue
         values = data[v]
         try:
             if len(unique_array(values)) == 1:
                 mapping[k] = values[0]
                 del data[v]
-        except:
+        except Exception:
             pass
 
 def cds_column_replace(source, data):
@@ -664,15 +786,15 @@ def hold_policy(document, policy, server=False):
     """
     Context manager to temporary override the hold policy.
     """
-    old_policy = document._hold
-    document._hold = policy
+    old_policy = document.callbacks.hold_value
+    document.callbacks._hold = policy
     try:
         yield
     finally:
         if server and not old_policy:
             document.unhold()
         else:
-            document._hold = old_policy
+            document.callbacks._hold = old_policy
 
 
 def recursive_model_update(model, props):
@@ -715,23 +837,40 @@ def update_shared_sources(f):
     def wrapper(self, *args, **kwargs):
         source_cols = self.handles.get('source_cols', {})
         shared_sources = self.handles.get('shared_sources', [])
+        doc = self.document
         for source in shared_sources:
             source.data.clear()
-            if self.document and self.document._held_events:
-                self.document._held_events = self.document._held_events[:-1]
+            if doc:
+                event_obj = doc.callbacks
+                event_obj._held_events = event_obj._held_events[:-1]
 
         ret = f(self, *args, **kwargs)
 
         for source in shared_sources:
             expected = source_cols[id(source)]
             found = [c for c in expected if c in source.data]
-            empty = np.full_like(source.data[found[0]], np.NaN) if found else []
+            empty = np.full_like(source.data[found[0]], np.nan) if found else []
             patch = {c: empty for c in expected if c not in source.data}
             source.data.update(patch)
         return ret
     return wrapper
 
 
+def hold_render(f):
+    """
+    Decorator that will hold render on a Bokeh ElementPlot until after
+    the method has been called.
+    """
+    def wrapper(self, *args, **kwargs):
+        hold = self.state.hold_render
+        self.state.hold_render = True
+        try:
+            return f(self, *args, **kwargs)
+        finally:
+            self.state.hold_render = hold
+    return wrapper
+
+
 def categorize_array(array, dim):
     """
     Uses a Dimension instance to convert an array of values to categorical
@@ -741,7 +880,7 @@ def categorize_array(array, dim):
     return np.array([dim.pprint_value(x) for x in array])
 
 
-class periodic(object):
+class periodic:
     """
     Mocks the API of periodic Thread in hv.core.util, allowing a smooth
     API transition on bokeh server.
@@ -771,7 +910,7 @@ class periodic(object):
     def __call__(self, period, count, callback, timeout=None, block=False):
         if isinstance(count, int):
             if count < 0: raise ValueError('Count value must be positive')
-        elif not type(count) is type(None):
+        elif type(count) is not type(None):
             raise ValueError('Count value must be a positive integer or None')
 
         self.callback = callback
@@ -802,9 +941,7 @@ class periodic(object):
         self._pcb = None
 
     def __repr__(self):
-        return 'periodic(%s, %s, %s)' % (self.period,
-                                         self.count,
-                                         callable_name(self.callback))
+        return f'periodic({self.period}, {self.count}, {callable_name(self.callback)})'
     def __str__(self):
         return repr(self)
 
@@ -814,8 +951,8 @@ def attach_periodic(plot):
     Attaches plot refresh to all streams on the object.
     """
     def append_refresh(dmap):
-        for dmap in get_nested_dmaps(dmap):
-            dmap.periodic._periodic_util = periodic(plot.document)
+        for subdmap in get_nested_dmaps(dmap):
+            subdmap.periodic._periodic_util = periodic(plot.document)
     return plot.hmap.traverse(append_refresh, [DynamicMap])
 
 
@@ -833,10 +970,10 @@ def date_to_integer(date):
     Returns:
         Milliseconds since 1970-01-01 00:00:00
     """
-    if pd and isinstance(date, pd.Timestamp):
+    if isinstance(date, pd.Timestamp):
         try:
             date = date.to_datetime64()
-        except:
+        except Exception:
             date = date.to_datetime()
 
     if isinstance(date, np.datetime64):
@@ -851,7 +988,7 @@ def date_to_integer(date):
     return dt_int
 
 
-def glyph_order(keys, draw_order=[]):
+def glyph_order(keys, draw_order=None):
     """
     Orders a set of glyph handles using regular sort and an explicit
     sort order. The explicit draw order must take the form of a list
@@ -859,6 +996,8 @@ def glyph_order(keys, draw_order=[]):
     suffix. The draw order may only match subset of the keys and any
     matched items will take precedence over other entries.
     """
+    if draw_order is None:
+        draw_order = []
     keys = sorted(keys)
     def order_fn(glyph):
         matches = [item for item in draw_order if glyph.startswith(item)]
@@ -940,18 +1079,67 @@ def match_dim_specs(specs1, specs2):
     return True
 
 
+def get_scale(range_input, axis_type):
+    if isinstance(range_input, (DataRange1d, Range1d)) and axis_type in ["linear", "datetime", "mercator", "auto", None]:
+        return LinearScale()
+    elif isinstance(range_input, (DataRange1d, Range1d)) and axis_type == "log":
+        return LogScale()
+    elif isinstance(range_input, FactorRange):
+        return CategoricalScale()
+    else:
+        raise ValueError(f"Unable to determine proper scale for: '{range_input}'")
+
+
+def get_axis_class(axis_type, range_input, dim): # Copied from bokeh
+    if axis_type is None:
+        return None, {}
+    elif axis_type == "linear":
+        return LinearAxis, {}
+    elif axis_type == "log":
+        return LogAxis, {}
+    elif axis_type == "datetime":
+        return DatetimeAxis, {}
+    elif axis_type == "mercator":
+        return MercatorAxis, dict(dimension='lon' if dim == 0 else 'lat')
+    elif axis_type == "auto":
+        if isinstance(range_input, FactorRange):
+            return CategoricalAxis, {}
+        elif isinstance(range_input, Range1d):
+            try:
+                value = range_input.start
+                # Datetime accepts ints/floats as timestamps, but we don't want
+                # to assume that implies a datetime axis
+                if Datetime.is_timestamp(value):
+                    return LinearAxis, {}
+                Datetime.validate(Datetime(), value)
+                return DatetimeAxis, {}
+            except ValueError:
+                pass
+        return LinearAxis, {}
+    else:
+        raise ValueError(f"Unrecognized axis_type: '{axis_type!r}'")
+
+
 def match_ax_type(ax, range_type):
     """
     Ensure the range_type matches the axis model being matched.
     """
-    if isinstance(ax[0], CategoricalAxis):
+    if isinstance(ax, CategoricalAxis):
         return range_type == 'categorical'
-    elif isinstance(ax[0], DatetimeAxis):
+    elif isinstance(ax, DatetimeAxis):
         return range_type == 'datetime'
     else:
         return range_type in ('auto', 'log')
 
 
+def match_yaxis_type_to_range(yax, range_type, range_name):
+    "Apply match_ax_type to the y-axis found by the given range name "
+    for axis in yax:
+        if axis.y_range_name == range_name:
+            return match_ax_type(axis, range_type)
+    raise ValueError('No axis with given range found')
+
+
 def wrap_formatter(formatter, axis):
     """
     Wraps formatting function or string in
@@ -959,14 +1147,67 @@ def wrap_formatter(formatter, axis):
     """
     if isinstance(formatter, TickFormatter):
         pass
-    elif isinstance(formatter, FunctionType):
-        msg = ('%sformatter could not be '
-               'converted to tick formatter. ' % axis)
-        jsfunc = py2js_tickformatter(formatter, msg)
-        if jsfunc:
-            formatter = FuncTickFormatter(code=jsfunc)
-        else:
-            formatter = None
     else:
         formatter = PrintfTickFormatter(format=formatter)
     return formatter
+
+
+def property_to_dict(x):
+    """
+    Convert Bokeh's property Field and Value to a dictionary
+    """
+
+    try:
+        from bokeh.core.property.vectorization import Field, Unspecified, Value
+
+        if isinstance(x, (Field, Value)):
+            x = {k: v for k, v in x.__dict__.items() if v != Unspecified}
+    except ImportError:
+        pass
+
+    return x
+
+
+def dtype_fix_hook(plot, element):
+    # Work-around for problems seen in:
+    # https://github.com/holoviz/holoviews/issues/5722
+    # https://github.com/holoviz/holoviews/issues/5726
+    # https://github.com/holoviz/holoviews/issues/5941
+    # Should be fixed in Bokeh:
+    # https://github.com/bokeh/bokeh/issues/13155
+
+    with suppress(Exception):
+        renderers = plot.handles["plot"].renderers
+        for renderer in renderers:
+            with suppress(Exception):
+                data = renderer.data_source.data
+                for k, v in data.items():
+                    if hasattr(v, "dtype") and v.dtype.kind == "U":
+                        data[k] = v.tolist()
+
+
+def get_ticker_axis_props(ticker):
+    axis_props = {}
+    if isinstance(ticker, np.ndarray):
+        ticker = list(ticker)
+    if isinstance(ticker, Ticker):
+        axis_props['ticker'] = ticker
+    elif isinstance(ticker, int):
+        axis_props['ticker'] = BasicTicker(desired_num_ticks=ticker)
+    elif isinstance(ticker, (tuple, list)):
+        if all(isinstance(t, tuple) for t in ticker):
+            ticks, labels = zip(*ticker)
+            # Ensure floats which are integers are serialized as ints
+            # because in JS the lookup fails otherwise
+            ticks = [int(t) if isinstance(t, float) and t.is_integer() else t
+                        for t in ticks]
+            labels = [l if isinstance(l, str) else str(l)
+                        for l in labels]
+        else:
+            ticks, labels = ticker, None
+        if ticks and util.isdatetime(ticks[0]):
+            ticks = [util.dt_to_int(tick, 'ms') for tick in ticks]
+        axis_props['ticker'] = FixedTicker(ticks=ticks)
+        if labels is not None:
+            axis_props['major_label_overrides'] = dict(zip(ticks, labels))
+    return axis_props
