@@ -2,13 +2,12 @@ from itertools import product
 
 import numpy as np
 import param
-
-from matplotlib.patches import Wedge, Circle
 from matplotlib.collections import LineCollection, PatchCollection
+from matplotlib.patches import Circle, Wedge
 
 from ...core.data import GridInterface
-from ...core.util import dimension_sanitizer, is_nan
 from ...core.spaces import HoloMap
+from ...core.util import dimension_sanitizer, is_nan
 from ..mixins import HeatMapMixin
 from .element import ColorbarPlot
 from .raster import QuadMeshPlot
@@ -174,8 +173,8 @@ class HeatMapPlot(HeatMapMixin, QuadMeshPlot):
             yvals = aggregate.dimension_values(ydim, expanded=False)
             yvals = GridInterface._infer_interval_breaks(yvals)
 
-        xfactors = list(ranges.get(xdim.name, {}).get('factors', []))
-        yfactors = list(ranges.get(ydim.name, {}).get('factors', []))
+        xfactors = list(ranges.get(xdim.label, {}).get('factors', []))
+        yfactors = list(ranges.get(ydim.label, {}).get('factors', []))
         xticks, yticks = self._compute_ticks(element, xvals, yvals, xfactors, yfactors)
 
         style['xfactors'] = xfactors
@@ -298,7 +297,7 @@ class RadialHeatMapPlot(ColorbarPlot):
         return ticks
 
 
-    def get_extents(self, view, ranges, range_type='combined'):
+    def get_extents(self, view, ranges, range_type='combined', **kwargs):
         if range_type == 'hard':
             return (np.nan,)*4
         return (0, 0, np.pi*2, self.max_radius+self.radius_outer)
