@@ -1,26 +1,41 @@
-from __future__ import absolute_import, division, unicode_literals
-
-from itertools import chain
 from contextlib import contextmanager
+from itertools import chain
 
-import param
-import numpy as np
 import matplotlib as mpl
-
-from mpl_toolkits.mplot3d import Axes3D  # noqa (For 3D plots)
-from matplotlib import pyplot as plt
-from matplotlib import gridspec, animation, rcParams
+import numpy as np
+import param
+from matplotlib import (
+    animation,
+    gridspec,
+    pyplot as plt,
+    rcParams,
+)
 from matplotlib.font_manager import font_scalings
+from mpl_toolkits.mplot3d import Axes3D  # noqa (For 3D plots)
 
-from ...core import (OrderedDict, HoloMap, AdjointLayout, NdLayout,
-                     GridSpace, Element, CompositeOverlay, Empty,
-                     Collator, GridMatrix, Layout)
-from ...core.options import Store, SkipRendering
-from ...core.util import int_to_roman, int_to_alpha, basestring, wrap_tuple_streams
-from ..plot import (DimensionedPlot, GenericLayoutPlot, GenericCompositePlot,
-                    GenericElementPlot, GenericAdjointLayoutPlot)
+from ...core import (
+    AdjointLayout,
+    Collator,
+    CompositeOverlay,
+    Element,
+    Empty,
+    GridMatrix,
+    GridSpace,
+    HoloMap,
+    Layout,
+    NdLayout,
+)
+from ...core.options import SkipRendering, Store
+from ...core.util import int_to_alpha, int_to_roman, wrap_tuple_streams
+from ..plot import (
+    DimensionedPlot,
+    GenericAdjointLayoutPlot,
+    GenericCompositePlot,
+    GenericElementPlot,
+    GenericLayoutPlot,
+)
 from ..util import attach_streams, collate, displayable
-from .util import compute_ratios, fix_aspect, mpl_version
+from .util import compute_ratios, fix_aspect, get_old_rcparams
 
 
 @contextmanager
@@ -28,15 +43,7 @@ def _rc_context(rcparams):
     """
     Context manager that temporarily overrides the pyplot rcParams.
     """
-    deprecated = [
-        'text.latex.unicode',
-        'examples.directory',
-        'savefig.frameon', # deprecated in MPL 3.1, to be removed in 3.3
-        'verbose.level', # deprecated in MPL 3.1, to be removed in 3.3
-        'verbose.fileo', # deprecated in MPL 3.1, to be removed in 3.3
-    ]
-    old_rcparams = {k: mpl.rcParams[k] for k in mpl.rcParams.keys()
-                    if mpl_version < '3.0' or k not in deprecated}
+    old_rcparams = get_old_rcparams()
     mpl.rcParams.clear()
     mpl.rcParams.update(dict(old_rcparams, **rcparams))
     try:
@@ -70,6 +77,13 @@ class MPLPlot(DimensionedPlot):
 
     sideplots = {}
 
+    backend_opts = param.Dict(default={}, doc="""
+        A dictionary of custom options to apply to the plot or
+        subcomponents of the plot. The keys in the dictionary mirror
+        attribute access on the underlying models stored in the plot's
+        handles, e.g. {'colorbar.margin': 10} will index the colorbar
+        in the Plot.handles and then set the margin to 10.""")
+
     fig_alpha = param.Number(default=1.0, bounds=(0, 1), doc="""
         Alpha of the overall figure background.""")
 
@@ -125,7 +139,7 @@ class MPLPlot(DimensionedPlot):
 
     def __init__(self, fig=None, axis=None, **params):
         self._create_fig = True
-        super(MPLPlot, self).__init__(**params)
+        super().__init__(**params)
         # List of handles to matplotlib objects for animation update
         self.fig_scale = self.fig_size/100.
         if isinstance(self.fig_inches, (tuple, list)):
@@ -255,7 +269,7 @@ class MPLPlot(DimensionedPlot):
 
 
     def update(self, key):
-        if len(self) == 1 and ((key == 0) or (key == self.keys[0])) and not self.drawn:
+        if len(self) == 1 and key in (0, self.keys[0]) and not self.drawn:
             return self.initialize_plot()
         return self.__getitem__(key)
 
@@ -290,7 +304,7 @@ class CompositePlot(GenericCompositePlot, MPLPlot):
         title = self._format_title(key) if self.show_title else ''
         if 'title' in self.handles:
             self.handles['title'].set_text(title)
-        else:
+        elif 'axis' in self.handles and self.handles['axis'].figure is not None:
             title = self.handles['axis'].set_title(title, **self._fontsize('title'))
             self.handles['title'] = title
 
@@ -346,8 +360,8 @@ class GridPlot(CompositePlot):
                  layout_num=1, keys=None, **params):
         if not isinstance(layout, GridSpace):
             raise Exception("GridPlot only accepts GridSpace.")
-        super(GridPlot, self).__init__(layout, layout_num=layout_num,
-                                       ranges=ranges, keys=keys, **params)
+        super().__init__(layout, layout_num=layout_num,
+                         ranges=ranges, keys=keys, **params)
         # Compute ranges layoutwise
         grid_kwargs = {}
         if axis is not None:
@@ -393,10 +407,10 @@ class GridPlot(CompositePlot):
         axiswise = all(norm_opts.get('axiswise', []))
         if not ranges:
             self.handles['fig'].set_size_inches(self.fig_inches)
-        subplots, subaxes = OrderedDict(), OrderedDict()
+        subplots, subaxes = {}, {}
         frame_ranges = self.compute_ranges(layout, None, ranges)
         keys = self.keys[:1] if self.dynamic else self.keys
-        frame_ranges = OrderedDict([(key, self.compute_ranges(layout, key, frame_ranges))
+        frame_ranges = dict([(key, self.compute_ranges(layout, key, frame_ranges))
                                     for key in keys])
         collapsed_layout = layout.clone(shared_data=False, id=layout.id)
         r, c = (0, 0)
@@ -486,7 +500,7 @@ class GridPlot(CompositePlot):
         # Get the extent of the layout elements (not the whole layout)
         key = self.keys[-1]
         axis = self.handles['axis']
-        subplot_kwargs = dict()
+        subplot_kwargs = {}
         ranges = self.compute_ranges(self.layout, key, ranges)
         for subplot in self.subplots.values():
             subplot.initialize_plot(ranges=ranges, **subplot_kwargs)
@@ -523,7 +537,7 @@ class GridPlot(CompositePlot):
         layout_axis.patch.set_visible(False)
 
         for ax, ax_obj in zip(['x', 'y'], [layout_axis.xaxis, layout_axis.yaxis]):
-            tick_fontsize = self._fontsize('%sticks' % ax,'labelsize', common=False)
+            tick_fontsize = self._fontsize(f'{ax}ticks','labelsize', common=False)
             if tick_fontsize: ax_obj.set_tick_params(**tick_fontsize)
 
         # Set labels
@@ -620,7 +634,7 @@ class GridPlot(CompositePlot):
             else:
                 r = 0
                 c += 1
-            if not ax is None:
+            if ax is not None:
                 ax.set_position([xpos, ypos, ax_w, ax_h])
 
 
@@ -647,8 +661,7 @@ class AdjointLayoutPlot(MPLPlot, GenericAdjointLayoutPlot):
 
         # The supplied (axes, view) objects as indexed by position
         self.subaxes = {pos: ax for ax, pos in zip(subaxes, self.view_positions)}
-        super(AdjointLayoutPlot, self).__init__(subplots=subplots, **params)
-
+        super().__init__(subplots=subplots, **params)
 
     @mpl_rc_context
     def initialize_plot(self, ranges=None):
@@ -673,7 +686,6 @@ class AdjointLayoutPlot(MPLPlot, GenericAdjointLayoutPlot):
         self.adjust_positions()
         self.drawn = True
 
-
     def adjust_positions(self, redraw=True):
         """
         Make adjustments to the positions of subplots (if available)
@@ -686,7 +698,7 @@ class AdjointLayoutPlot(MPLPlot, GenericAdjointLayoutPlot):
         checks = [self.view_positions, self.subaxes, self.subplots]
         right = all('right' in check for check in checks)
         top = all('top' in check for check in checks)
-        if not 'main' in self.subplots or not (top or right):
+        if "main" not in self.subplots or not (top or right):
             return
         if redraw:
             self.handles['fig'].canvas.draw()
@@ -721,7 +733,6 @@ class AdjointLayoutPlot(MPLPlot, GenericAdjointLayoutPlot):
             if isinstance(subplot, GridPlot):
                 ax.set_aspect('equal')
 
-
     @mpl_rc_context
     def update_frame(self, key, ranges=None):
         for pos in self.view_positions:
@@ -729,7 +740,6 @@ class AdjointLayoutPlot(MPLPlot, GenericAdjointLayoutPlot):
             if subplot is not None:
                 subplot.update_frame(key, ranges)
 
-
     def __len__(self):
         return max([1 if self.keys is None else len(self.keys), 1])
 
@@ -775,14 +785,13 @@ class LayoutPlot(GenericLayoutPlot, CompositePlot):
     v17_layout_format = True
 
     def __init__(self, layout, keys=None, **params):
-        super(LayoutPlot, self).__init__(layout=layout, keys=keys, **params)
+        super().__init__(layout=layout, keys=keys, **params)
         with mpl.rc_context(rc=self.fig_rcparams):
             self.subplots, self.subaxes, self.layout = self._compute_gridspec(layout)
         if self.top_level:
             self.traverse(lambda x: attach_streams(self, x.hmap, 2),
                           [GenericElementPlot])
 
-
     def _compute_gridspec(self, layout):
         """
         Computes the tallest and widest cell for each row and column
@@ -791,7 +800,7 @@ class LayoutPlot(GenericLayoutPlot, CompositePlot):
         appropriate embedded layout_types. The first element of the
         returned tuple is a dictionary of all the LayoutPlots indexed
         by row and column. The second dictionary in the tuple supplies
-        the grid indicies needed to instantiate the axes for each
+        the grid indices needed to instantiate the axes for each
         LayoutPlot.
         """
         layout_items = layout.grid_items()
@@ -815,7 +824,7 @@ class LayoutPlot(GenericLayoutPlot, CompositePlot):
             main = layout_view.main
             main = main.last if isinstance(main, HoloMap) else main
             main_options = self.lookup_options(main, 'plot').options if main else {}
-            if main and not isinstance(main_options.get('aspect', 1), basestring):
+            if main and not isinstance(main_options.get('aspect', 1), str):
                 main_aspect = np.nan if isinstance(main, Empty) else main_options.get('aspect', 1)
                 main_aspect = self.aspect_weight*main_aspect + 1-self.aspect_weight
             else:
@@ -833,7 +842,7 @@ class LayoutPlot(GenericLayoutPlot, CompositePlot):
             else:
                 width_ratios = [4]
 
-            inv_aspect = 1./main_aspect if main_aspect else np.NaN
+            inv_aspect = 1./main_aspect if main_aspect else np.nan
             if layout_type in ['Embedded Dual', 'Triple']:
                 el = layout_view.get('top', None)
                 eltype = type(el)
@@ -846,7 +855,7 @@ class LayoutPlot(GenericLayoutPlot, CompositePlot):
             else:
                 height_ratios = [4]
 
-            if not isinstance(main_aspect, (basestring, type(None))):
+            if not isinstance(main_aspect, (str, type(None))):
                 width_ratios = [wratio * main_aspect for wratio in width_ratios]
                 height_ratios = [hratio * inv_aspect for hratio in height_ratios]
             layout_shape = (len(width_ratios), len(height_ratios))
@@ -906,6 +915,10 @@ class LayoutPlot(GenericLayoutPlot, CompositePlot):
                                     wspace=self.hspace,
                                     hspace=self.vspace)
 
+        # Explicitly clear Matplotlib figures to avoid
+        # "Auto-removal of overlapping axes" warning.
+        self.handles['fig'].clf()
+
         # Situate all the Layouts in the grid and compute the gridspec
         # indices for all the axes required by each LayoutPlot.
         gidx = 0
@@ -914,7 +927,7 @@ class LayoutPlot(GenericLayoutPlot, CompositePlot):
         collapsed_layout = layout.clone(shared_data=False, id=layout.id)
         frame_ranges = self.compute_ranges(layout, None, None)
         keys = self.keys[:1] if self.dynamic else self.keys
-        frame_ranges = OrderedDict([(key, self.compute_ranges(layout, key, frame_ranges))
+        frame_ranges = dict([(key, self.compute_ranges(layout, key, frame_ranges))
                                     for key in keys])
         layout_subplots, layout_axes = {}, {}
         for r, c in self.coords:
@@ -942,7 +955,7 @@ class LayoutPlot(GenericLayoutPlot, CompositePlot):
 
             layout_key, _ = layout_items.get((r, c), (None, None))
             if isinstance(layout, NdLayout) and layout_key:
-                layout_dimensions = OrderedDict(zip(layout_dimensions, layout_key))
+                layout_dimensions = dict(zip(layout_dimensions, layout_key))
 
             # Generate the axes and create the subplots with the appropriate
             # axis objects, handling any Empty objects.
@@ -953,7 +966,7 @@ class LayoutPlot(GenericLayoutPlot, CompositePlot):
             elif empty:
                 obj = AdjointLayout([])
             elif not view.traverse(lambda x: x, [Element]):
-                self.param.warning('%s is empty, skipping subplot.' % obj.main)
+                self.param.warning(f'{obj.main} is empty, skipping subplot.')
                 continue
             elif self.transpose:
                 layout_count = (c*self.rows+(r+1))
@@ -990,7 +1003,6 @@ class LayoutPlot(GenericLayoutPlot, CompositePlot):
 
         return layout_subplots, layout_axes, collapsed_layout
 
-
     def grid_situate(self, current_idx, layout_type, subgrid_width):
         """
         Situate the current AdjointLayoutPlot in a LayoutPlot. The
@@ -1024,8 +1036,7 @@ class LayoutPlot(GenericLayoutPlot, CompositePlot):
 
         return start, inds
 
-
-    def _create_subplots(self, layout, positions, layout_dimensions, ranges, axes={}, num=1, create=True):
+    def _create_subplots(self, layout, positions, layout_dimensions, ranges, axes=None, num=1, create=True):
         """
         Plot all the views contained in the AdjointLayout Object using axes
         appropriate to the layout configuration. All the axes are
@@ -1033,6 +1044,8 @@ class LayoutPlot(GenericLayoutPlot, CompositePlot):
         invoke subplots with correct options and styles and hide any
         empty axes as necessary.
         """
+        if axes is None:
+            axes = {}
         subplots = {}
         projections = []
         adjoint_clone = layout.clone(shared_data=False, id=layout.id)
@@ -1060,7 +1073,7 @@ class LayoutPlot(GenericLayoutPlot, CompositePlot):
             override_opts = {}
             sublabel_opts = {}
             if pos == 'main':
-                own_params = self.param.get_param_values(onlychanged=True)
+                own_params = self.param.values(onlychanged=True).items()
                 sublabel_opts = {k: v for k, v in own_params
                                  if 'sublabel_' in k}
             elif pos == 'right':
@@ -1113,8 +1126,10 @@ class LayoutPlot(GenericLayoutPlot, CompositePlot):
         if (not self.traverse(specs=[GridPlot]) and not isinstance(self.fig_inches, tuple)
             and self.v17_layout_format):
             traverse_fn = lambda x: x.handles.get('bbox_extra_artists', None)
-            extra_artists = list(chain(*[artists for artists in self.traverse(traverse_fn)
-                                         if artists is not None]))
+            extra_artists = list(
+                chain.from_iterable(artists for artists in self.traverse(traverse_fn)
+                                    if artists is not None)
+            )
             fix_aspect(fig, self.rows, self.cols,
                        title_obj, extra_artists,
                        vspace=self.vspace*self.fig_scale,
@@ -1123,8 +1138,8 @@ class LayoutPlot(GenericLayoutPlot, CompositePlot):
             for cbar_plot in colorbars:
                 if cbar_plot.colorbar:
                     cbar_plot._draw_colorbar(redraw=False)
-            adjoined = self.traverse(specs=[AdjointLayoutPlot])
-            for adjoined in adjoined:
+            adjoineds = self.traverse(specs=[AdjointLayoutPlot])
+            for adjoined in adjoineds:
                 if len(adjoined.subplots) > 1:
                     adjoined.adjust_positions(redraw=False)
         return self._finalize_axis(None)
