@@ -1,47 +1,46 @@
-# -*- coding: utf-8 -*-
 """
 Public API for all plotting renderers supported by HoloViews,
 regardless of plotting package or backend.
 """
-from __future__ import unicode_literals, absolute_import
-
 import base64
 import os
-
-from io import BytesIO
-try:
-    from StringIO import StringIO
-except ImportError:
-    from io import StringIO
 from contextlib import contextmanager
 from functools import partial
+from io import BytesIO, StringIO
 
+import panel as pn
 import param
-import panel
-
 from bokeh.document import Document
-from bokeh.io import curdoc
 from bokeh.embed import file_html
+from bokeh.io import curdoc
 from bokeh.resources import CDN, INLINE
+from packaging.version import Version
 from panel import config
-from panel.io.notebook import ipywidget, load_notebook, render_model, render_mimebundle
+from panel.io.notebook import (
+    JupyterCommManagerBinary,
+    ipywidget,
+    load_notebook,
+    render_mimebundle,
+    render_model,
+)
 from panel.io.state import state
+from panel.models.comm_manager import CommManager as PnCommManager
 from panel.pane import HoloViews as HoloViewsPane
-from panel.widgets.player import PlayerBase
 from panel.viewable import Viewable
-from pyviz_comms import CommManager, JupyterCommManager
+from panel.widgets.player import PlayerBase
+from param.parameterized import bothmethod
+from pyviz_comms import CommManager
 
-from ..core import Layout, HoloMap, AdjointLayout, DynamicMap
+from ..core import AdjointLayout, DynamicMap, HoloMap, Layout
+from ..core.data import disable_pipeline
 from ..core.io import Exporter
-from ..core.options import Store, StoreOptions, SkipRendering, Compositor
-from ..core.util import basestring, unbound_dimensions, LooseVersion
+from ..core.options import Compositor, SkipRendering, Store, StoreOptions
+from ..core.util import unbound_dimensions
 from ..streams import Stream
 from . import Plot
-from .util import displayable, collate, initialize_dynamic
+from .util import collate, displayable, initialize_dynamic
 
-from param.parameterized import bothmethod
-
-panel_version = LooseVersion(panel.__version__)
+panel_version = Version(pn.__version__)
 
 # Tags used when visual output is to be embedded in HTML
 IMAGE_TAG = "<img src='{src}' style='max-width:100%; margin: auto; display: block; {css}'/>"
@@ -108,16 +107,16 @@ class Renderer(Exporter):
 
     backend = param.String(doc="""
         The full, lowercase name of the rendering backend or third
-        part plotting package used e.g 'matplotlib' or 'cairo'.""")
+        part plotting package used e.g. 'matplotlib' or 'cairo'.""")
 
-    dpi = param.Integer(None, doc="""
+    dpi = param.Integer(default=None, doc="""
         The render resolution in dpi (dots per inch)""")
 
     fig = param.ObjectSelector(default='auto', objects=['auto'], doc="""
         Output render format for static figures. If None, no figure
         rendering will occur. """)
 
-    fps = param.Number(20, doc="""
+    fps = param.Number(default=20, doc="""
         Rendered fps (frames per second) for animated formats.""")
 
     holomap = param.ObjectSelector(default='auto',
@@ -131,7 +130,7 @@ class Renderer(Exporter):
         mode a bokeh Document will be returned which can be served as a
         bokeh server app. By default renders all output is rendered to HTML.""")
 
-    size = param.Integer(100, doc="""
+    size = param.Integer(default=100, doc="""
         The rendered size as a percentage size""")
 
     widget_location = param.ObjectSelector(default=None, allow_None=True, objects=[
@@ -147,10 +146,10 @@ class Renderer(Exporter):
     css = param.Dict(default={}, doc="""
         Dictionary of CSS attributes and values to apply to HTML output.""")
 
-    info_fn = param.Callable(None, allow_None=True, constant=True,  doc="""
+    info_fn = param.Callable(default=None, allow_None=True, constant=True,  doc="""
         Renderers do not support the saving of object info metadata""")
 
-    key_fn = param.Callable(None, allow_None=True, constant=True,  doc="""
+    key_fn = param.Callable(default=None, allow_None=True, constant=True,  doc="""
         Renderers do not support the saving of object key metadata""")
 
     post_render_hooks = param.Dict(default={'svg':[], 'png':[]}, doc="""
@@ -183,8 +182,7 @@ class Renderer(Exporter):
 
     def __init__(self, **params):
         self.last_plot = None
-        super(Renderer, self).__init__(**params)
-
+        super().__init__(**params)
 
     def __call__(self, obj, fmt='auto', **kwargs):
         plot, fmt = self._validate(obj, fmt)
@@ -201,14 +199,13 @@ class Renderer(Exporter):
             data = self._apply_post_render_hooks(data, obj, fmt)
             return data, info
 
-
     @bothmethod
     def get_plot(self_or_cls, obj, doc=None, renderer=None, comm=None, **kwargs):
         """
         Given a HoloViews Viewable return a corresponding plot instance.
         """
         if isinstance(obj, DynamicMap) and obj.unbounded:
-            dims = ', '.join('%r' % dim for dim in obj.unbounded)
+            dims = ', '.join(f'{dim!r}' for dim in obj.unbounded)
             msg = ('DynamicMap cannot be displayed without explicit indexing '
                    'as {dims} dimension(s) are unbounded. '
                    '\nSet dimensions bounds with the DynamicMap redim.range '
@@ -227,7 +224,9 @@ class Renderer(Exporter):
             if not displayable(obj):
                 obj = collate(obj)
                 initialize_dynamic(obj)
-            obj = Compositor.map(obj, mode='data', backend=self_or_cls.backend)
+
+            with disable_pipeline():
+                obj = Compositor.map(obj, mode='data', backend=self_or_cls.backend)
             plot_opts = dict(self_or_cls.plot_options(obj, self_or_cls.size),
                              **kwargs)
             if isinstance(obj, AdjointLayout):
@@ -241,6 +240,18 @@ class Renderer(Exporter):
         else:
             plot = obj
 
+        # Trigger streams which were marked as requiring an update
+        triggers = []
+        for p in plot.traverse():
+            if not hasattr(p, '_trigger'):
+                continue
+            for trigger in p._trigger:
+                if trigger not in triggers:
+                    triggers.append(trigger)
+            p._trigger = []
+        for trigger in triggers:
+            Stream.trigger([trigger])
+
         if isinstance(self_or_cls, Renderer):
             self_or_cls.last_plot = plot
 
@@ -253,17 +264,15 @@ class Renderer(Exporter):
             plot.document = doc
         return plot
 
-
     @bothmethod
     def get_plot_state(self_or_cls, obj, renderer=None, **kwargs):
         """
         Given a HoloViews Viewable return a corresponding plot state.
         """
         if not isinstance(obj, Plot):
-            obj = self_or_cls.get_plot(obj, renderer, **kwargs)
+            obj = self_or_cls.get_plot(obj=obj, renderer=renderer, **kwargs)
         return obj.state
 
-
     def _validate(self, obj, fmt, **kwargs):
         """
         Helper method to be used in the __call__ method to get a
@@ -293,19 +302,17 @@ class Renderer(Exporter):
             plot = self.get_widget(obj, fmt)
             fmt = 'html'
         elif dynamic or (self._render_with_panel and fmt == 'html'):
-            plot = HoloViewsPane(obj, center=True, backend=self.backend,
+            plot = HoloViewsPane(obj, center=self.center, backend=self.backend,
                                  renderer=self)
         else:
             plot = self.get_plot(obj, renderer=self, **kwargs)
 
         all_formats = set(fig_formats + holomap_formats)
         if fmt not in all_formats:
-            raise Exception("Format %r not supported by mode %r. Allowed formats: %r"
-                            % (fmt, self.mode, fig_formats + holomap_formats))
+            raise Exception(f"Format {fmt!r} not supported by mode {self.mode!r}. Allowed formats: {fig_formats + holomap_formats!r}")
         self.last_plot = plot
         return plot, fmt
 
-
     def _apply_post_render_hooks(self, data, obj, fmt):
         """
         Apply the post-render hooks to the data.
@@ -315,11 +322,10 @@ class Renderer(Exporter):
             try:
                 data = hook(data, obj)
             except Exception as e:
-                self.param.warning("The post_render_hook %r could not "
-                                   "be applied:\n\n %s" % (hook, e))
+                self.param.warning(f"The post_render_hook {hook!r} could not "
+                                   f"be applied:\n\n {e}")
         return data
 
-
     def html(self, obj, fmt=None, css=None, resources='CDN', **kwargs):
         """
         Renders plot or data structure and wraps the output in HTML.
@@ -328,7 +334,7 @@ class Renderer(Exporter):
         """
         plot, fmt =  self._validate(obj, fmt)
         figdata, _ = self(plot, fmt, **kwargs)
-        if isinstance(resources, basestring):
+        if isinstance(resources, str):
             resources = resources.lower()
         if css is None: css = self.css
 
@@ -342,15 +348,14 @@ class Renderer(Exporter):
             return file_html(doc, resources)
         elif fmt in ['html', 'json']:
             return figdata
-        else:
-            if fmt == 'svg':
-                figdata = figdata.encode("utf-8")
-            elif fmt == 'pdf' and 'height' not in css:
-                _, h = self.get_size(plot)
-                css['height'] = '%dpx' % (h*self.dpi*1.15)
+        elif fmt == 'svg':
+            figdata = figdata.encode("utf-8")
+        elif fmt == 'pdf' and 'height' not in css:
+            _, h = self.get_size(plot)
+            css['height'] = '%dpx' % (h*self.dpi*1.15)
 
         if isinstance(css, dict):
-            css = '; '.join("%s: %s" % (k, v) for k, v in css.items())
+            css = '; '.join(f"{k}: {v}" for k, v in css.items())
         else:
             raise ValueError("CSS must be supplied as Python dictionary")
 
@@ -360,7 +365,6 @@ class Renderer(Exporter):
         html = tag.format(src=src, mime_type=mime_type, css=css)
         return html
 
-
     def components(self, obj, fmt=None, comm=True, **kwargs):
         """
         Returns data and metadata dictionaries containing HTML and JS
@@ -372,70 +376,65 @@ class Renderer(Exporter):
         else:
             plot, fmt = self._validate(obj, fmt)
 
-        data, metadata = {}, {}
-        if isinstance(plot, Viewable):
-            registry = list(Stream.registry.items())
-            objects = plot.object.traverse(lambda x: x)
-            dynamic, streams = False, False
-            for source in objects:
-                dynamic |= isinstance(source, DynamicMap)
-                streams |= any(
-                    src is source or (src._plot_id is not None and src._plot_id == source._plot_id)
-                    for src, streams in registry for s in streams
-                )
-            embed = (not (dynamic or streams or self.widget_mode == 'live') or config.embed)
-
-            # This part should be factored out in Panel and then imported
-            # here for HoloViews 2.0, which will be able to require a
-            # recent Panel version.
-            if embed or config.comms == 'default':
-                comm = self.comm_manager.get_server_comm() if comm else None
-                doc = Document()
-                with config.set(embed=embed):
-                    model = plot.layout._render_model(doc, comm)
-                if embed:
-                    return render_model(model, comm)
-                args = (model, doc, comm)
-                if panel_version > '0.9.3':
-                    from panel.models.comm_manager import CommManager
-                    ref = model.ref['id']
-                    manager = CommManager(comm_id=comm.id, plot_id=ref)
-                    client_comm = self.comm_manager.get_client_comm(
-                        on_msg=partial(plot._on_msg, ref, manager),
-                        on_error=partial(plot._on_error, ref),
-                        on_stdout=partial(plot._on_stdout, ref)
-                    )
-                    manager.client_comm_id = client_comm.id
-                    args = args + (manager,)
-                return render_mimebundle(*args)
-
-            # Handle rendering object as ipywidget
-            widget = ipywidget(plot, combine_events=True)
-            if hasattr(widget, '_repr_mimebundle_'):
-                return widget._repr_mimebundle()
-            plaintext = repr(widget)
-            if len(plaintext) > 110:
-                plaintext = plaintext[:110] + '…'
-            data = {
-                'text/plain': plaintext,
-            }
-            if widget._view_name is not None:
-                data['application/vnd.jupyter.widget-view+json'] = {
-                    'version_major': 2,
-                    'version_minor': 0,
-                    'model_id': widget._model_id
-                }
-            if config.comms == 'vscode':
-                # Unfortunately VSCode does not yet handle _repr_mimebundle_
-                from IPython.display import display
-                display(data, raw=True)
-                return {'text/html': '<div style="display: none"></div>'}, {}
-            return data, {}
-        else:
+        if not isinstance(plot, Viewable):
             html = self._figure_data(plot, fmt, as_script=True, **kwargs)
-        data['text/html'] = html
+            return {'text/html': html}, {MIME_TYPES['jlab-hv-exec']: {}}
 
-        return (data, {MIME_TYPES['jlab-hv-exec']: metadata})
+        registry = list(Stream.registry.items())
+        objects = plot.object.traverse(lambda x: x)
+        dynamic, streams = False, False
+        for source in objects:
+            dynamic |= isinstance(source, DynamicMap)
+            streams |= any(
+                src is source or (src._plot_id is not None and src._plot_id == source._plot_id)
+                for src, streams in registry for s in streams
+            )
+        if config.comms == 'colab':
+            load_notebook(config.inline)
+        embed = (not (dynamic or streams or self.widget_mode == 'live') or config.embed)
+        if embed or config.comms == 'default':
+            return self._render_panel(plot, embed, comm)
+        return self._render_ipywidget(plot)
+
+    def _render_panel(self, plot, embed=False, comm=True):
+        comm = self.comm_manager.get_server_comm() if comm else None
+        doc = Document()
+        with config.set(embed=embed):
+            model = plot.layout._render_model(doc, comm)
+        if embed:
+            return render_model(model, comm)
+        ref = model.ref['id']
+        manager = PnCommManager(comm_id=comm.id, plot_id=ref)
+        client_comm = self.comm_manager.get_client_comm(
+            on_msg=partial(plot._on_msg, ref, manager),
+            on_error=partial(plot._on_error, ref),
+            on_stdout=partial(plot._on_stdout, ref),
+            on_open=lambda _: comm.init()
+        )
+        manager.client_comm_id = client_comm.id
+        return render_mimebundle(model, doc, comm, manager)
+
+    def _render_ipywidget(self, plot):
+        # Handle rendering object as ipywidget
+        widget = ipywidget(plot, combine_events=True)
+        if hasattr(widget, '_repr_mimebundle_'):
+            return widget._repr_mimebundle_(), {}
+        plaintext = repr(widget)
+        if len(plaintext) > 110:
+            plaintext = plaintext[:110] + '…'
+        data = {'text/plain': plaintext}
+        if widget._view_name is not None:
+            data['application/vnd.jupyter.widget-view+json'] = {
+                'version_major': 2,
+                'version_minor': 0,
+                'model_id': widget._model_id
+            }
+        if config.comms == 'vscode':
+            # Unfortunately VSCode does not yet handle _repr_mimebundle_
+            from IPython.display import display
+            display(data, raw=True)
+            return {'text/html': '<div style="display: none"></div>'}, {}
+        return data, {}
 
     def static_html(self, obj, fmt=None, template=None):
         """
@@ -448,7 +447,6 @@ class Renderer(Exporter):
         html_bytes.seek(0)
         return html_bytes.read()
 
-
     @bothmethod
     def get_widget(self_or_cls, plot, widget_type, **kwargs):
         if widget_type == 'scrubber':
@@ -464,7 +462,6 @@ class Renderer(Exporter):
             player.interval = interval
         return layout
 
-
     @bothmethod
     def export_widgets(self_or_cls, obj, filename, fmt=None, template=None,
                        json=False, json_path='', **kwargs):
@@ -481,7 +478,6 @@ class Renderer(Exporter):
                              "registered widget types.")
         self_or_cls.get_widget(obj, fmt).save(filename)
 
-
     @bothmethod
     def _widget_kwargs(self_or_cls):
         if self_or_cls.holomap in ('auto', 'widgets'):
@@ -492,7 +488,6 @@ class Renderer(Exporter):
             loc = self_or_cls.widget_location or 'bottom'
         return {'widget_location': loc, 'widget_type': widget_type, 'center': True}
 
-
     @bothmethod
     def app(self_or_cls, plot, show=False, new_window=False, websocket_origin=None, port=0):
         """
@@ -518,7 +513,6 @@ class Renderer(Exporter):
             kwargs = {'notebook_url': websocket_origin} if websocket_origin else {}
             return pane.app(port=port, **kwargs)
 
-
     @bothmethod
     def server_doc(self_or_cls, obj, doc=None):
         """
@@ -531,7 +525,6 @@ class Renderer(Exporter):
                                 **self_or_cls._widget_kwargs())
         return obj.layout.server_doc(doc)
 
-
     @classmethod
     def plotting_class(cls, obj):
         """
@@ -544,23 +537,14 @@ class Renderer(Exporter):
             element_type = obj
         else:
             element_type = obj.type if isinstance(obj, HoloMap) else type(obj)
+            if element_type is None:
+                raise SkipRendering(f"{type(obj).__name__} was empty, could not determine plotting class.")
         try:
             plotclass = Store.registry[cls.backend][element_type]
         except KeyError:
-            raise SkipRendering("No plotting class for {0} "
-                                "found".format(element_type.__name__))
+            raise SkipRendering(f"No plotting class for {element_type.__name__} found.") from None
         return plotclass
 
-
-    @classmethod
-    def html_assets(cls, core=True, extras=True, backends=None, script=False):
-        """
-        Deprecated: No longer needed
-        """
-        param.main.warning("Renderer.html_assets is deprecated as all "
-                           "JS and CSS dependencies are now handled by "
-                           "Panel.")
-
     @classmethod
     def plot_options(cls, obj, percent_size):
         """
@@ -573,19 +557,22 @@ class Renderer(Exporter):
         """
         raise NotImplementedError
 
-
     @bothmethod
-    def save(self_or_cls, obj, basename, fmt='auto', key={}, info={},
+    def save(self_or_cls, obj, basename, fmt='auto', key=None, info=None,
              options=None, resources='inline', title=None, **kwargs):
         """
         Save a HoloViews object to file, either using an explicitly
         supplied format or to the appropriate default.
         """
+        if info is None:
+            info = {}
+        if key is None:
+            key = {}
         if info or key:
             raise Exception('Renderer does not support saving metadata to file.')
 
         if kwargs:
-            param.main.warning("Supplying plot, style or norm options "
+            param.main.param.warning("Supplying plot, style or norm options "
                                "as keyword arguments to the Renderer.save "
                                "method is deprecated and will error in "
                                "the next minor release.")
@@ -601,11 +588,11 @@ class Renderer(Exporter):
                 resources = CDN
             elif resources.lower() == 'inline':
                 resources = INLINE
-            if isinstance(basename, basestring):
+            if isinstance(basename, str):
                 if title is None:
                     title = os.path.basename(basename)
                 if fmt in MIME_TYPES:
-                    basename = '.'.join([basename, fmt])
+                    basename = f"{basename}.{fmt}"
             plot.layout.save(basename, embed=True, resources=resources, title=title)
             return
 
@@ -620,7 +607,7 @@ class Renderer(Exporter):
             basename.write(encoded)
             basename.seek(0)
         else:
-            filename ='%s.%s' % (basename, info['file-ext'])
+            filename =f"{basename}.{info['file-ext']}"
             with open(filename, 'wb') as f:
                 f.write(encoded)
 
@@ -629,7 +616,6 @@ class Renderer(Exporter):
         "Hook to prefix content for instance JS when saving HTML"
         return
 
-
     @bothmethod
     def get_size(self_or_cls, plot):
         """
@@ -650,7 +636,6 @@ class Renderer(Exporter):
         """
         yield
 
-
     @classmethod
     def validate(cls, options):
         """
@@ -658,25 +643,30 @@ class Renderer(Exporter):
         """
         return options
 
-
     @classmethod
-    def load_nb(cls, inline=True):
+    def load_nb(cls, inline=False, reloading=False, enable_mathjax=False):
         """
         Loads any resources required for display of plots
         in the Jupyter notebook
         """
-        load_notebook(inline)
+        if panel_version >= Version('1.0.2'):
+            load_notebook(inline, reloading=reloading, enable_mathjax=enable_mathjax)
+        elif panel_version >= Version('1.0.0'):
+            load_notebook(inline, reloading=reloading)
+        elif reloading:
+            return
+        else:
+            load_notebook(inline)
         with param.logging_level('ERROR'):
             try:
                 ip = get_ipython() # noqa
-            except:
+            except Exception:
                 ip = None
             if not ip or not hasattr(ip, 'kernel'):
                 return
             cls.notebook_context = True
-            cls.comm_manager = JupyterCommManager
-            state._comm_manager = JupyterCommManager
-
+            cls.comm_manager = JupyterCommManagerBinary
+            state._comm_manager = JupyterCommManagerBinary
 
     @classmethod
     def _delete_plot(cls, plot_id):
