@@ -6,13 +6,15 @@ from unittest import SkipTest
 
 import pandas as pd
 import param
+import pytest
 from panel.widgets import IntSlider
 
+import holoviews as hv
 from holoviews.core.spaces import DynamicMap
-from holoviews.core.util import LooseVersion
-from holoviews.element import Points, Scatter, Curve, Histogram, Polygons
+from holoviews.core.util import NUMPY_GE_200, Version
+from holoviews.element import Curve, Histogram, Points, Polygons, Scatter
 from holoviews.element.comparison import ComparisonTestCase
-from holoviews.streams import * # noqa (Test all available streams)
+from holoviews.streams import *  # noqa (Test all available streams)
 from holoviews.util import Dynamic, extension
 from holoviews.util.transform import dim
 
@@ -23,18 +25,17 @@ def test_all_stream_parameters_constant():
     all_stream_cls = [v for v in globals().values() if
                       isinstance(v, type) and issubclass(v, Stream)]
     for stream_cls in all_stream_cls:
-        for name, p in stream_cls.param.params().items():
+        for name, p in stream_cls.param.objects().items():
             if name == 'name': continue
             if p.constant != True:
-                raise TypeError('Parameter %s of stream %s not declared constant'
-                                % (name, stream_cls.__name__))
+                raise TypeError(f'Parameter {name} of stream {stream_cls.__name__} not declared constant')
 
 
 def test_all_linked_stream_parameters_owners():
     "Test to ensure operations can accept parameters in streams dictionary"
     stream_classes = param.concrete_descendents(LinkedStream)
     for stream_class in stream_classes.values():
-        for name, p in stream_class.param.params().items():
+        for name, p in stream_class.param.objects().items():
             if name != 'name' and (p.owner != stream_class):
                 msg = ("Linked stream %r has parameter %r which is "
                        "inherited from %s. Parameter needs to be redeclared "
@@ -72,30 +73,42 @@ class TestStreamsDefine(ComparisonTestCase):
         self.assertEqual(xy.y, 2)
 
     def test_XY_set_invalid_class_x(self):
-        regexp = "Parameter 'x' only takes numeric values"
+        if Version(param.__version__) > Version('2.0.0a2'):
+            regexp = "Number parameter 'XY.x' only takes numeric values"
+        else:
+            regexp = "Parameter 'x' only takes numeric values"
         with self.assertRaisesRegex(ValueError, regexp):
             self.XY.x = 'string'
 
     def test_XY_set_invalid_class_y(self):
-        regexp = "Parameter 'y' only takes numeric values"
+        if Version(param.__version__) > Version('2.0.0a2'):
+            regexp = "Number parameter 'XY.y' only takes numeric values"
+        else:
+            regexp = "Parameter 'y' only takes numeric values"
         with self.assertRaisesRegex(ValueError, regexp):
             self.XY.y = 'string'
 
     def test_XY_set_invalid_instance_x(self):
         xy = self.XY(x=1,y=2)
-        regexp = "Parameter 'x' only takes numeric values"
+        if Version(param.__version__) > Version('2.0.0a2'):
+            regexp = "Number parameter 'XY.x' only takes numeric values"
+        else:
+            regexp = "Parameter 'x' only takes numeric values"
         with self.assertRaisesRegex(ValueError, regexp):
             xy.x = 'string'
 
     def test_XY_set_invalid_instance_y(self):
         xy = self.XY(x=1,y=2)
-        regexp = "Parameter 'y' only takes numeric values"
+        if Version(param.__version__) > Version('2.0.0a2'):
+            regexp = "Number parameter 'XY.y' only takes numeric values"
+        else:
+            regexp = "Parameter 'y' only takes numeric values"
         with self.assertRaisesRegex(ValueError, regexp):
             xy.y = 'string'
 
     def test_XY_subscriber_triggered(self):
 
-        class Inner(object):
+        class Inner:
             def __init__(self): self.state=None
             def __call__(self, x,y): self.state=(x,y)
 
@@ -120,7 +133,7 @@ class TestStreamsDefine(ComparisonTestCase):
         self.assertEqual(self.ExplicitTest.param['test'].doc, 'Test docstring')
 
 
-class _TestSubscriber(object):
+class _TestSubscriber:
 
     def __init__(self, cb=None):
         self.call_count = 0
@@ -169,7 +182,7 @@ class TestParamsStream(LoggingComparisonTestCase):
 
         class InnerAction(Inner):
 
-            action = param.Action(lambda o: o.param.trigger('action'))
+            action = param.Action(default=lambda o: o.param.trigger('action'))
 
         self.inner = Inner
         self.inner_action = InnerAction
@@ -274,7 +287,7 @@ class TestParamsStream(LoggingComparisonTestCase):
         def subscriber(**kwargs):
             values.append(kwargs)
             self.assertEqual(set(stream.hashkey),
-                             {'%s action' % id(inner), '_memoize_key'})
+                             {f'{id(inner)} action', '_memoize_key'})
 
         stream.add_subscriber(subscriber)
         inner.action(inner)
@@ -290,7 +303,7 @@ class TestParamsStream(LoggingComparisonTestCase):
             values.append(kwargs)
             self.assertEqual(
                 set(stream.hashkey),
-                {'%s action' % id(inner), '%s x' % id(inner), '_memoize_key'})
+                {f'{id(inner)} action', f'{id(inner)} x', '_memoize_key'})
 
         stream.add_subscriber(subscriber)
         inner.action(inner)
@@ -330,12 +343,12 @@ class TestParamsStream(LoggingComparisonTestCase):
 class TestParamMethodStream(ComparisonTestCase):
 
     def setUp(self):
-        if LooseVersion(param.__version__) < LooseVersion('1.8.0'):
+        if Version(param.__version__) < Version('1.8.0'):
             raise SkipTest('Params stream requires param >= 1.8.0')
 
         class Inner(param.Parameterized):
 
-            action = param.Action(lambda o: o.param.trigger('action'))
+            action = param.Action(default=lambda o: o.param.trigger('action'))
             x = param.Number(default = 0)
             y = param.Number(default = 0)
             count = param.Integer(default=0)
@@ -424,11 +437,8 @@ class TestParamMethodStream(ComparisonTestCase):
         self.assertEqual(dmap[()], Points([10]))
 
     def test_panel_param_steams_dict(self):
-        try:
-            import panel
-        except:
-            raise SkipTest('Panel required for widget support in streams dict')
-        widget = panel.widgets.FloatSlider(value=1)
+        import panel as pn
+        widget = pn.widgets.FloatSlider(value=1)
 
         def test(x):
             return Points([x])
@@ -531,7 +541,7 @@ class TestParamMethodStream(ComparisonTestCase):
         def subscriber(**kwargs):
             values.append(kwargs)
             self.assertEqual(set(stream.hashkey),
-                             {'%s action' % id(inner), '_memoize_key'})
+                             {f'{id(inner)} action', '_memoize_key'})
 
         stream.add_subscriber(subscriber)
         inner.action(inner)
@@ -551,7 +561,7 @@ class TestParamMethodStream(ComparisonTestCase):
             values.append(kwargs)
             self.assertEqual(
                 set(stream.hashkey),
-                {'%s action' % id(inner), '%s x' % id(inner), '_memoize_key'})
+                {f'{id(inner)} action', f'{id(inner)} x', '_memoize_key'})
 
         stream.add_subscriber(subscriber)
         stream.add_subscriber(lambda **kwargs: dmap[()])
@@ -587,6 +597,24 @@ class TestParamMethodStream(ComparisonTestCase):
         self.assertEqual(values_y, [{}])
 
 
+@pytest.mark.usefixtures("bokeh_backend")
+def test_dynamicmap_partial_bind_and_streams():
+    # Ref: https://github.com/holoviz/holoviews/issues/6008
+
+    def make_plot(z, x_range, y_range):
+        return Curve([1, 2, 3, 4, z])
+
+    slider = IntSlider(name='Slider', start=0, end=10)
+    range_xy = RangeXY()
+
+    dmap = DynamicMap(param.bind(make_plot, z=slider), streams=[range_xy])
+
+    bk_figure = hv.render(dmap)
+
+    assert bk_figure.renderers[0].data_source.data["y"][-1] == 0
+    assert range_xy.x_range == (0, 4)
+    assert range_xy.y_range == (-0.4, 4.4)
+
 
 class TestSubscribers(ComparisonTestCase):
 
@@ -784,24 +812,26 @@ class TestPlotSizeTransform(ComparisonTestCase):
 class TestPipeStream(ComparisonTestCase):
 
     def test_pipe_send(self):
-        test = None
         def subscriber(data):
-            global test
-            test = data
+            subscriber.test = data
+        subscriber.test = None
+
         pipe = Pipe()
         pipe.add_subscriber(subscriber)
         pipe.send('Test')
         self.assertEqual(pipe.data, 'Test')
+        self.assertEqual(subscriber.test, 'Test')
 
     def test_pipe_event(self):
-        test = None
         def subscriber(data):
-            global test
-            test = data
+            subscriber.test = data
+        subscriber.test = None
+
         pipe = Pipe()
         pipe.add_subscriber(subscriber)
         pipe.event(data='Test')
         self.assertEqual(pipe.data, 'Test')
+        self.assertEqual(subscriber.test, 'Test')
 
     def test_pipe_update(self):
         pipe = Pipe()
@@ -845,7 +875,7 @@ class TestBufferArrayStream(ComparisonTestCase):
 
     def test_buffer_array_send_verify_shape_fail(self):
         buff = Buffer(np.array([[0, 1]]))
-        error = "Streamed array data expeced to have 2 columns, got 3."
+        error = "Streamed array data expected to have 2 columns, got 3."
         with self.assertRaisesRegex(ValueError, error):
             buff.send(np.array([[1, 2, 3]]))
 
@@ -1123,8 +1153,9 @@ class TestExprSelectionStream(ComparisonTestCase):
     def setUp(self):
         extension("bokeh")
 
-    def test_selection_expr_stream_scatter_points(self):
-        for element_type in [Scatter, Points]:
+    def test_selection_expr_stream_2D_elements(self):
+        element_type_2D = [Points]
+        for element_type in element_type_2D:
             # Create SelectionExpr on element
             element = element_type(([1, 2, 3], [1, 5, 10]))
             expr_stream = SelectionExpr(element)
@@ -1150,8 +1181,36 @@ class TestExprSelectionStream(ComparisonTestCase):
                 {'x': (1, 3), 'y': (1, 4)}
             )
 
-    def test_selection_expr_stream_invert_axes(self):
-        for element_type in [Scatter, Points]:
+    def test_selection_expr_stream_1D_elements(self):
+        element_type_1D = [Scatter]
+        for element_type in element_type_1D:
+            # Create SelectionExpr on element
+            element = element_type(([1, 2, 3], [1, 5, 10]))
+            expr_stream = SelectionExpr(element)
+
+            # Check stream properties
+            self.assertEqual(len(expr_stream.input_streams), 1)
+            self.assertIsInstance(expr_stream.input_streams[0], SelectionXY)
+            self.assertIsNone(expr_stream.bbox)
+            self.assertIsNone(expr_stream.selection_expr)
+
+            # Simulate interactive update by triggering source stream
+            expr_stream.input_streams[0].event(bounds=(1, 1, 3, 4))
+
+            # Check SelectionExpr values
+            self.assertEqual(
+                repr(expr_stream.selection_expr),
+                repr((dim('x')>=1)&(dim('x')<=3))
+            )
+            self.assertEqual(
+                expr_stream.bbox,
+                {'x': (1, 3)}
+            )
+
+
+    def test_selection_expr_stream_invert_axes_2D_elements(self):
+        element_type_2D = [Points]
+        for element_type in element_type_2D:
             # Create SelectionExpr on element
             element = element_type(([1, 2, 3], [1, 5, 10])).opts(invert_axes=True)
             expr_stream = SelectionExpr(element)
@@ -1177,8 +1236,35 @@ class TestExprSelectionStream(ComparisonTestCase):
                 {'y': (1, 3), 'x': (1, 4)}
             )
 
-    def test_selection_expr_stream_invert_xaxis_yaxis(self):
-        for element_type in [Scatter, Points]:
+    def test_selection_expr_stream_invert_axes_1D_elements(self):
+        element_type_1D = [Scatter]
+        for element_type in element_type_1D:
+            # Create SelectionExpr on element
+            element = element_type(([1, 2, 3], [1, 5, 10])).opts(invert_axes=True)
+            expr_stream = SelectionExpr(element)
+
+            # Check stream properties
+            self.assertEqual(len(expr_stream.input_streams), 1)
+            self.assertIsInstance(expr_stream.input_streams[0], SelectionXY)
+            self.assertIsNone(expr_stream.bbox)
+            self.assertIsNone(expr_stream.selection_expr)
+
+            # Simulate interactive update by triggering source stream
+            expr_stream.input_streams[0].event(bounds=(1, 1, 3, 4))
+
+            # Check SelectionExpr values
+            self.assertEqual(
+                repr(expr_stream.selection_expr),
+                repr((dim('x')>=1)&(dim('x')<=4))
+            )
+            self.assertEqual(
+                expr_stream.bbox,
+                {'x': (1, 4)}
+            )
+
+    def test_selection_expr_stream_invert_xaxis_yaxis_2D_elements(self):
+        element_type_2D = [Points]
+        for element_type in element_type_2D:
 
             # Create SelectionExpr on element
             element = element_type(([1, 2, 3], [1, 5, 10])).opts(
@@ -1208,6 +1294,36 @@ class TestExprSelectionStream(ComparisonTestCase):
                 {'x': (1, 3), 'y': (1, 4)}
             )
 
+    def test_selection_expr_stream_invert_xaxis_yaxis_1D_elements(self):
+        element_type_1D = [Scatter]
+        for element_type in element_type_1D:
+
+            # Create SelectionExpr on element
+            element = element_type(([1, 2, 3], [1, 5, 10])).opts(
+                invert_xaxis=True,
+                invert_yaxis=True,
+            )
+            expr_stream = SelectionExpr(element)
+
+            # Check stream properties
+            self.assertEqual(len(expr_stream.input_streams), 1)
+            self.assertIsInstance(expr_stream.input_streams[0], SelectionXY)
+            self.assertIsNone(expr_stream.bbox)
+            self.assertIsNone(expr_stream.selection_expr)
+
+            # Simulate interactive update by triggering source stream
+            expr_stream.input_streams[0].event(bounds=(3, 4, 1, 1))
+
+            # Check SelectionExpr values
+            self.assertEqual(
+                repr(expr_stream.selection_expr),
+                repr((dim('x')>=1)&(dim('x')<=3))
+            )
+            self.assertEqual(
+                expr_stream.bbox,
+                {'x': (1, 3)}
+            )
+
     def test_selection_expr_stream_hist(self):
         # Create SelectionExpr on element
         hist = Histogram(([1, 2, 3, 4, 5], [1, 5, 2, 3, 7]))
@@ -1305,11 +1421,12 @@ class TestExprSelectionStream(ComparisonTestCase):
 
 
     def test_selection_expr_stream_polygon_index_cols(self):
+        # TODO: Should test both spatialpandas and shapely
         # Create SelectionExpr on element
         try: import shapely # noqa
-        except:
+        except ImportError:
             try: import spatialpandas # noqa
-            except: raise SkipTest('Shapely required for polygon selection')
+            except ImportError: raise SkipTest('Shapely required for polygon selection')
         poly = Polygons([
             [(0, 0, 'a'), (2, 0, 'a'), (1, 1, 'a')],
             [(2, 0, 'b'), (4, 0, 'b'), (3, 1, 'b')],
@@ -1328,10 +1445,12 @@ class TestExprSelectionStream(ComparisonTestCase):
         self.assertIsNone(expr_stream.bbox)
         self.assertIsNone(expr_stream.selection_expr)
 
+        fmt = lambda x: list(map(np.str_, x)) if NUMPY_GE_200 else x
+
         expr_stream.input_streams[2].event(index=[0, 1])
         self.assertEqual(
             repr(expr_stream.selection_expr),
-            repr(dim('cat').isin(['a', 'b']))
+            repr(dim('cat').isin(fmt(['a', 'b'])))
         )
         self.assertEqual(expr_stream.bbox, None)
         self.assertEqual(len(events), 1)
@@ -1340,7 +1459,7 @@ class TestExprSelectionStream(ComparisonTestCase):
         expr_stream.input_streams[0].event(bounds=(0, 0, 4, 1))
         self.assertEqual(
             repr(expr_stream.selection_expr),
-            repr(dim('cat').isin(['a', 'b']))
+            repr(dim('cat').isin(fmt(['a', 'b'])))
         )
         self.assertEqual(len(events), 1)
 
@@ -1348,7 +1467,7 @@ class TestExprSelectionStream(ComparisonTestCase):
         expr_stream.input_streams[1].event(geometry=np.array([(0, 0), (4, 0), (4, 2), (0, 2)]))
         self.assertEqual(
             repr(expr_stream.selection_expr),
-            repr(dim('cat').isin(['a', 'b', 'c']))
+            repr(dim('cat').isin(fmt(['a', 'b', 'c'])))
         )
         self.assertEqual(len(events), 2)
 
@@ -1356,13 +1475,14 @@ class TestExprSelectionStream(ComparisonTestCase):
         expr_stream.input_streams[2].event(index=[1, 2])
         self.assertEqual(
             repr(expr_stream.selection_expr),
-            repr(dim('cat').isin(['b', 'c']))
+            repr(dim('cat').isin(fmt(['b', 'c'])))
         )
         self.assertEqual(expr_stream.bbox, None)
         self.assertEqual(len(events), 3)
 
-    def test_selection_expr_stream_dynamic_map(self):
-        for element_type in [Scatter, Points]:
+    def test_selection_expr_stream_dynamic_map_2D_elements(self):
+        element_type_2D = [Points]
+        for element_type in element_type_2D: # Scatter,
             # Create SelectionExpr on element
             dmap = Dynamic(element_type(([1, 2, 3], [1, 5, 10])))
             expr_stream = SelectionExpr(dmap)
@@ -1385,3 +1505,29 @@ class TestExprSelectionStream(ComparisonTestCase):
                 expr_stream.bbox,
                 {'x': (1, 3), 'y': (1, 4)}
             )
+
+    def test_selection_expr_stream_dynamic_map_1D_elements(self):
+        element_type_1D = [Scatter]
+        for element_type in element_type_1D:
+            # Create SelectionExpr on element
+            dmap = Dynamic(element_type(([1, 2, 3], [1, 5, 10])))
+            expr_stream = SelectionExpr(dmap)
+
+            # Check stream properties
+            self.assertEqual(len(expr_stream.input_streams), 1)
+            self.assertIsInstance(expr_stream.input_streams[0], SelectionXY)
+            self.assertIsNone(expr_stream.bbox)
+            self.assertIsNone(expr_stream.selection_expr)
+
+            # Simulate interactive update by triggering source stream
+            expr_stream.input_streams[0].event(bounds=(1, 1, 3, 4))
+
+            # Check SelectionExpr values
+            self.assertEqual(
+                repr(expr_stream.selection_expr),
+                repr((dim('x')>=1)&(dim('x')<=3))
+            )
+            self.assertEqual(
+                expr_stream.bbox,
+                {'x': (1, 3)}
+            )
